//----------------------------------------------------------------------------------------------------------------------------------------------------------//
//									LEEM Image Analysis											//
//																								// 
//										F. Cheynis												//
//																								//
//										03/05/2016												//
//----------------------------------------------------------------------------------------------------------------------------------------------------------//

// 2016-05-03 Version: GUI updated and left/right browsing buttons added
// 2016-03-29 Version: Killwaves added in Extract procedure & Overlayers parsing updated including Obj. lens, Pressure and FOV
// 2014-05-23 Version : MyImageHeader.ColorScaleLow and MyImageHeader.ColorScaleHigh can be modified by Background corrections. This is implementd in DAVWriteFrame. 
// 2012-10-15 Version : Remove Backrground implemented
// 2011-01-24 Version : Overlayer data can be included or not in the "convert" procedure
// 2010-11-24 Version : dat Sequences 
// 2010-01-29 Version : Drift correction pre-scan


#include "LEEM_Particle Extensions2009"
#include "LEEM_ROI Extensions2009"
#include "LEEM_Drift Correction2009"
#include "LEEM_Remove Background2009"
//#include "aaa_MovieMakers"

#pragma rtGlobals=1		// Use modern global access method.

Menu "Macros"
	"LEEM Analysis 2009", LeemPanel2009()
	//"-",
End

Window LeemPanel2009() : Panel

	Variable /g  FrameNumber=0,t_i=0,t_0=0,MovFileLoaded=0
	Variable /g FromVar, ToVar, StepVar, DurationVar, DoesCursorAExist,PolyOrderVar
	String/G YesNoStr,OptionStr, BlackStr, ManualStr,FormatStr,OverlayerStr,XcursorAStr,YcursorAStr
	
	//newpath/Q/O MyPath, "C:Users:Fabien:Documents:Recherche:Dmouillage:LEEM"
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=1 /W=(696,62,1032,337) as "LEEM 2009"
	ModifyPanel cbRGB=(56576,56576,56576)
	SetDrawLayer UserBack
	SetDrawEnv fsize= 11,textrgb= (0,15872,65280)
	DrawText 158,275,"Rev. Sci. Instrum., 85 (2014) 043705"
	GroupBox FrameGroupBox,pos={6,208},size={322,51},title="\\K(0,15872,65280)Single Frames"
	GroupBox FrameGroupBox,labelBack=(48896,52992,65280),font="Arial",frame=0
	GroupBox MOVGroupBox,pos={6,147},size={322,51},title="\\F'Arial'\\K(0,15872,65280).mov Files"
	GroupBox MOVGroupBox,labelBack=(48896,52992,65280),frame=0
	GroupBox DAVGroupBox,pos={6,1},size={322,139},title="\\K(0,15872,65280).dav Files and .dat Sequences"
	GroupBox DAVGroupBox,labelBack=(48896,52992,65280),font="Arial",frame=0
	Button DAVOpenButton,pos={19,27},size={65,22},proc=SeqOpen2009,title="Open .dav"
	Button DAVOpenButton,font="Arial"
	Button DATSeqOpenButton,pos={180,27},size={65,22},proc=SeqOpen2009,title="Open Seq"
	Button DATSeqOpenButton,font="Arial"
	Button DAVLoadButton,pos={263,21},size={60,35},proc=DAVLoad2009,title="Load\rFrame"
	Button DAVLoadButton,font="Arial"
	SetVariable DAVSetvar,pos={116,30},size={30,16},title=" "
	SetVariable DAVSetvar,help={"Index of the frame to be loaded (>=0)"}
	SetVariable DAVSetvar,limits={0,200,0},value= FrameNumber,styledText= 1
	Slider DAVslider,pos={28,58},size={218,52},proc=DAVsliderProc2009
	Slider DAVslider,limits={0,166,1},variable= FrameNumber,vert= 0,ticks= 10
	Button LoadImageButton,pos={82,232},size={79,22},proc=Load2009,title="Load Image"
	Button LoadImageButton,font="Arial"
	PopupMenu ImageLoadPopup,pos={199,232},size={47,21},title="\\F'Arial'"
	PopupMenu ImageLoadPopup,mode=3,popvalue="dat",value= #"\"dat;tiff;png;jpeg\""
	Button LoadMOVButton,pos={128,168},size={79,22},proc=LoadMov,title="Load .mov"
	Button LoadMOVButton,font="Arial"
	Button DAVExtractButton,pos={265,62},size={60,20},proc=DAVExtract2009,title="Extract"
	Button DAVExtractButton,font="Arial"
	Button DAVConvertButton,pos={265,88},size={60,20},proc=DAVConvert2009,title="Convert"
	Button DAVConvertButton,font="Arial"
	Button DAVDriftButton,pos={102,115},size={60,20},proc=DAVDrift2009,title="Drift Corr."
	Button DAVDriftButton,font="Arial"
	Button DAVPartButton,pos={183,115},size={65,20},proc=DAVPart2009,title="Part. Anal."
	Button DAVPartButton,font="Arial"
	Button DAVROIButton,pos={265,115},size={60,20},proc=DAVROI2009,title="I\\B ROI"
	Button DAVROIButton,font="Arial"
	Button DAVBackgroundButton,pos={21,115},size={60,20},proc=DAVBackground2009,title="Backgrnd"
	Button DAVBackgroundButton,font="Arial"
	Button DAVSliderButtonRight,pos={245,56},size={19,20},title="\\W649",proc=DAVSliderButton2009
	Button DAVSliderButtonLeft,pos={9,57},size={20,19},title="\\W646",proc=DAVSliderButton2009
EndMacro



//----------------------------------------------------------------------------------------------------------------------------------------------------//
//								DAV Files and DAT Sequences								//
//----------------------------------------------------------------------------------------------------------------------------------------------------//

//defined in LEEM_Analysis2015
//Structure LONGLONG
//	uint32 LONG[2]
//EndStructure


Structure UKFileHeader2009
	char id[20]
	int16 size
	int16 version
	int16 BitsPerPixel
	uchar filler[6]	
	double spare
	int16 ImageWidth, ImageHeight
	int16 NrImages
	int16 attachedRecipeSize
	uchar SeqRecipe[56]	  
EndStructure

Structure UKImageHeader2009
	int16 size
	int16 version
	int16 ColorScaleLow,ColorScaleHigh
	STRUCT LONGLONG imagetime
	int16 MaskXshift, MaskYshift
	uchar useMask
	uchar spare
	int16 attachedMarkupSize
	int16 spin
	int16 LEEMdataVersion
	uchar LEEMdata[256]
	uchar filler[4]			  
EndStructure

//Dialog to open a .dav file, Generation of the first image
//Function DAVOpen(ctrlName) : ButtonControl
//	String ctrlName
//	
//	Variable/G RefRead=0,NFrames=0
//	//Nvar V_logEOF
//	close/a
//	Setdatafolder root:
//	open/R /F="Data Files (*.dav):.dav;All Files:.*;" RefRead
//	fstatus RefRead
//	If(V_flag==0)
//		print "Open procedure failed"
//	Else
//		Nvar MovFileLoaded
//		MovFileLoaded=0
//		NFrames=floor((V_logEOF-104)/(288+1024*1024*2))
//		If(NFrames==1)
//			Slider DAVslider, disable=1 
//		Else
//			SetVariable DAVSetvar, limits={0,NFrames-1,0}, noedit=0
//			Slider DAVslider, limits={0,NFrames-1,1} ,disable=0
//		Endif
//		DAVLoadFrame(0,1)
//		If(Exists("M_ROIMask")&&Exists("M_ROIMask_boundary"))
//			KillWaves/Z M_ROIMask, M_ROIMask_boundary
//		EndIf
//		WMCreateImageROIPanel()
//	EndIf
//End

//Dialog to open a .dat sequence, Generation of the first image
Function SeqOpen2009(ctrlName) : ButtonControl
	String ctrlName
	
	Variable/G RefRead=0,NFrames=0
	
	Nvar FrameNumber; FrameNumber=0
	//Nvar V_logEOF
	
	//newpath/Q/O MyPath, "V:demouillage:SOI_0per:SOI-0per-LEEM-10"
	String ExtractOverlaysStr
	
	close/a
	Setdatafolder root:
	If(StringMatch(ctrlName,"*DAV*"))
		//open/R/P=MyPath /F="Data Files (*.dav):.dav;All Files:.*;" RefRead
		open/R /F="Data Files (*.dav):.dav;All Files:.*;" RefRead		
	Else
		//open/R/P=MyPath /F="Data Files (*.dat):.dat;All Files:.*;" RefRead
		open/R /F="Data Files (*.dat):.dat;All Files:.*;" RefRead
	EndIf
	
	fstatus RefRead
	If(V_flag==0)
		abort "Open procedure failed"
	Else
		Print "Loading Sequence "+S_path+S_fileName
	Endif
		
	Nvar MovFileLoaded
	MovFileLoaded=0
	If(StringMatch(ctrlName,"*DAV*"))
		NFrames=floor((V_logEOF-104)/(288+1024*1024*2))
	Else
		String/G DATSeqRootFilename,DATSeqFilenameList
		String TmpStr, BuildADavS
		Variable i, NDATFiles
		
		NewPath/Q/Z/O DATSeqPath, S_path
		SplitString/E="\\D+" S_fileName
		DATSeqRootFilename=S_value
		//Print DATSeqRootFilename
		TmpStr=IndexedFile(DATSeqPath, -1, ".dat")
		NDATFiles=ItemsInList(TmpStr)
		//print TmpStr
		
		for(i=0;i<NDATFiles; i+=1)
			if(!StringMatch(StringFromList(i,TmpStr),"*"+DATSeqRootFilename+"*"))
				TmpStr=RemoveListItem(i,TmpStr)
				i-=1
				NDATFiles-=1
			EndIf
		EndFor
							
		//print TmpStr, NDATFiles
		//Print RemoveFromList("*LEED_000000*", TmpStr, ";")
		//print TmpStr
		DATSeqFilenameList=TmpStr
		NFrames=NDATFiles
		
		Prompt BuildADavS,"Do you want to build a .dav file from the current .dat sequence", popup "Yes;No"
		DoPrompt "",BuildADavS
		if(V_flag==0)			
			if(stringmatch(BuildADavS,"Yes"))
				DAVBuild2009()
			Endif
		Endif
	EndIf
	
	If(NFrames==1)
		Slider DAVslider, disable=1, win=LeemPanel2009
	Else
		SetVariable DAVSetvar, limits={0,NFrames-1,0}, noedit=0,win=LeemPanel2009
		Slider DAVslider, limits={0,NFrames-1,1} ,disable=0,win=LeemPanel2009
	Endif
	
	DAVLoadFrame2009(0,1)
	
	//Clean potential ROIMask waves from previous sequence
	If(Exists("M_ROIMask"))
		KillWaves/Z M_ROIMask
	EndIf
	If(Exists("M_ROIMask_boundary"))
		KillWaves/Z MM_ROIMask_boundary
	EndIf
	
	
	WMCreateImageROIPanel()
	Nvar DoesCursorAExist;DoesCursorAExist=0 //Initialise the flag telling if cursor A is on the graph
	
	//Extract data overlays in dedicated waves Temperature, Pressure, Time, ...
	Prompt ExtractOverlaysStr,"Do you want to extract the sequence overlays ?", popup "No;Yes"
	DoPrompt "", ExtractOverlaysStr
	if(V_flag==0)			
		if(stringmatch(ExtractOverlaysStr,"Yes"))
			make/O/N=(NFrames) SampleTempW, PressureW, ElapsedTimeW,StartVoltW
			SampleTempW=nan; PressureW=nan; ElapsedTimeW=nan; StartVoltW=nan
			For(i=0;i<NFrames; i+=1)
				DAVOverlays2009(i)
			Endfor			
		Endif
	Endif
	
	
	
End

Function DAVLoad2009(ctrlName) : ButtonControl
	String ctrlName
	Nvar FrameNumber, RefRead 
	
	Fstatus RefRead
	// Loads a .dat image
	If(StringMatch(S_filename, "*.dat*"))
		Svar DATSeqFilenameList
		Nvar NFrames
		String Str
		
		Close/a
		Str=StringFromList(FrameNumber,DATSeqFilenameList)
		open/R /P=DATSeqPath RefRead as Str
		
		DAVLoadFrame2009(0,1)
	Else
	// Loads an image from a .dav sequence
		DAVLoadFrame2009(FrameNumber,1)
	EndIf	
End

Function/WAVE DAVLoadFrame2009(FrameNr,DisplayMode)
	
	Variable FrameNr, DisplayMode
	
	Nvar RefRead, t_i, t_0
	Nvar FrameNumber//; FrameNumber=FrameNr
	String/G FrameName, FileHeader, ImageHeader
	
	Variable Temp=0, SV=0, Pressure=0, ElapsedTime=0, i, TempVar,Obj=0
	String FOV="", Str
		
	STRUCT UKFileHeader2009 MyFileHeader 
	STRUCT UKImageHeader2009 MyImageHeader
	
	
	
	fsetpos RefRead,0
	FBinRead RefRead, MyFileHeader
	
	Fstatus RefRead	
	
	If(StringMatch(S_filename, "*.dat*"))
		// for .dat sequence we have to open the appropriate file
		If(FrameNr!=0)
			Svar DATSeqFilenameList
			Close/a
			Str=StringFromList(FrameNr,DATSeqFilenameList)
			open/R /P=DATSeqPath RefRead as Str
		EndIf
		
		FrameNr=0
		// File header: 104 bytes and Image header:288 bytes		
		fsetpos RefRead,104+(0)*(288+1024*1024*2)
		FBinRead RefRead, MyImageHeader
		
		// .dat files can have some markups hence the offset of 128 bytes
		If(MyImageHeader.attachedMarkupSize==0)
			//no markups
			fsetpos RefRead, 104+(288+0)*(0+1)+(0)*1024*1024*2
		Else		
			//128 byte markup block after image header	
			fsetpos RefRead, 104+(288+128)*(0+1)+(0)*1024*1024*2
		EndIf	
	Else
		//for .dav files: no markups
		fsetpos RefRead,104+(FrameNr)*(288+1024*1024*2)
		FBinRead RefRead, MyImageHeader
		
		fsetpos RefRead, 104+288*(FrameNr+1)+(FrameNr)*1024*1024*2	
	EndIf	
		
	FrameName="Frame"+num2str(FrameNr)
	Print FrameName
	make/D/O/N=(1024,1024) $FrameName /WAVE=w	
	
	FBinRead/f=2 RefRead, w
	//duplicate/O w, $FrameName
	//KillWaves w
		
	fstatus RefRead
	If(DisplayMode==1)
		t_i=0				
		
		t_i=MyImageHeader.imagetime.LONG[0]
		t_i+=2^32*MyImageHeader.imagetime.LONG[1]
			
		If(FrameNr==0)
			t_0=t_i
		Else
			If(t_0==0)			
				ElapsedTime=0
			Else
				ElapsedTime=(t_i-t_0)*1e-7 //Elapsed  time (from t_0 to t_i) converted from 100ns to s
			EndIf
		EndIf
		
		t_i*=1e-7 // t_i converted from 100ns to s
		t_i-=9561628800 // t_i converted from Windows Filetime format (01/01/1601) to Mac Epoch format (01/01/1970)
		
		//To understand LEEMdata structure and debug LEEMdata parsing
		//For(i=0;i<256;i+=1)
		//	print MyImageHeader.LEEMdata[i] 
		//EndFor
		
		For(i=0;i<256;i+=1)
			//39 is the ascii number of the module 'Sample Temp.'
			If(MyImageHeader.LEEMdata[i]==39)
				//File header:104 bytes+28 bytes before LEEMdata block in the Image header+i+15 bytes to reach Temp. data
				fsetpos RefRead,104+28+i+15+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, Temp
				// To reach the end-1 of this block
				i+=18
			EndIf
			//38 is the ascii number of the module 'Start Voltage'
			If(MyImageHeader.LEEMdata[i]==38)
				fsetpos RefRead,104+28+i+16+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, SV
				// To reach the end-1 of this block
				i+=19
			EndIf
			//110 is the ascii number of the module FOV but it's also the character 'n'
			If(MyImageHeader.LEEMdata[i]==110)
				//To make sure we are not in the 'Emission Cur.' block or in the 'Main'
				If(MyImageHeader.LEEMdata[i+1]!=32 && MyImageHeader.LEEMdata[i+1]!=0)
					//FOV=LEED
					If(MyImageHeader.LEEMdata[i+1]==76)
						FOV="LEED"
						// To reach the end-1 of this block
						i+=9
					Else
						FOV=num2char(MyImageHeader.LEEMdata[i+1])
						FOV+=num2char(MyImageHeader.LEEMdata[i+2])
						FOV+=num2char(MyImageHeader.LEEMdata[i+3])
						FOV+=num2char(MyImageHeader.LEEMdata[i+4])
						//Print "FOV i=",i
						//print "LEEM", MyImageHeader.LEEMdata[i+1]
												
						//3 digit FOV:
						If(MyImageHeader.LEEMdata[i+6]==42)	
							FOV+=num2char(MyImageHeader.LEEMdata[i+5])
							//Print "FOV i=",i
							//print "LEEM", MyImageHeader.LEEMdata[i+1]
							i+=1
						EndIf
						i+=10
					EndIf	
				EndIf
			EndIf
			//106 is the ascii number of the module 'Main'
			If(MyImageHeader.LEEMdata[i]==106)
				//Sometimes the unit 'Torr' (ascii code 'T' is 84) is inserted between the two 'NULL' characters
				//Sometimes there's a space character (ascii code is 32)!!
				If(MyImageHeader.LEEMdata[i+6]==32)
					fsetpos RefRead,104+28+i+8+(FrameNr)*(288+1024*1024*2)
					FBinRead/f=4 RefRead, Pressure
					i+=11
				elseif(MyImageHeader.LEEMdata[i+6]==84)
					fsetpos RefRead,104+28+i+11+(FrameNr)*(288+1024*1024*2)
					FBinRead/f=4 RefRead, Pressure
					i+=14
				Endif
			EndIf	
				
			//11 is the ascii number of the module Obj
			If(MyImageHeader.LEEMdata[i]==11)
				fsetpos RefRead,104+28+i+12+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, Obj
				i+=5
			EndIf
			
		EndFor
	//	fsetpos RefRead,104+43+(FrameNr)*(288+1024*1024*2)
	//	FBinRead/f=4 RefRead, Temp
	//	fsetpos RefRead,104+63+(FrameNr)*(288+1024*1024*2)
	//	FBinRead/f=4 RefRead, SV
	//	FOV=num2char(MyImageHeader.LEEMdata[40])
	//	FOV+=num2char(MyImageHeader.LEEMdata[41])
	//	fsetpos RefRead,104+89+(FrameNr)*(288+1024*1024*2)
	//	FBinRead/f=4 RefRead, Pressure
	//	Str=num2str(MyImageHeader.LEEMdata[0])
	//	For(i=1;i<256;i+=1)
	//		Str+=" "
	//		Str+=num2str(MyImageHeader.LEEMdata[i])
	//		//Print MyImageHeader.LEEMdata[i]
	//	EndFor
	//	Print Str	
	
		Display/T/K=1/W=(0, 0, 512, 512 ); AppendImage/T/G=0 $FrameName; ModifyGraph margin=-1
		//NewImage/K=1/S=0/G=0 $FrameName
		AutoPositionWindow/E/M=1/R=LeemPanel2009
		//ModifyImage $FrameName ctab= {MyImageHeader.ColorScaleLow,MyImageHeader.ColorScaleHigh,Grays}
		ModifyImage $FrameName ctab= {*,*,Grays},log=1
		
		If(StringMatch(S_fileName,"*.dav*"))
			TextBox/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName+" "+FrameName
		Else
			TextBox/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName
		EndIf
		TextBox/N=Day/F=0/B=1/A=LT/X=2/Y=93"\\Z10\\K(65280,65280,0)"+secs2date(t_i,0)+" "+secs2time(t_i,1)
		TextBox/N=Ell/F=0/B=1/A=LT/X=2/Y=96"\\Z10\\K(65280,65280,0)Elapsed time="+num2str(ElapsedTime)+"s"
		TextBox/N=Temp/F=0/B=1/A=LT/X=79/Y=10"\\Z10\\K(65280,65280,0) T\Bsample\M\\Z10="+num2str(Temp)+"C"
		TextBox/N=Press/F=0/B=1/A=LT/X=79/Y=7"\\Z10\\K(65280,65280,0) P="+num2str(Pressure)+"Torr"
		TextBox/N=FOV/F=0/B=1/A=LT/X=79/Y=93"\\Z10\\K(65280,65280,0) FOV="+FOV//+"m"
		TextBox/N=SV/F=0/B=1/A=LT/X=79/Y=90 "\\Z10\\K(65280,65280,0) SV="+num2str(SV)+"V"
		TextBox/N=Obj/F=0/B=1/A=LT/X=79/Y=96"\\Z10\\K(65280,65280,0) Obj.="+num2str(Obj)+"mA"
		
		//Adjust Image Contrast if "WMContrastAdjustGraph" Window is loaded 
		Str=WinList("*",";","WIN:1")
		if(StringMatch(Str,"*WMContrastAdjustGraph*"))
			SVAR ImGrfName=root:Packages:WMImProcess:ImageContrast:ImGrfName
			ImGrfName=WinName(0,1,1)
			WMContrastButtonProc("Apply")
			
			Str=WaveList("*_CLU",";","")
			KillWaves/Z Str
		EndIf
		
	EndIf		
	//close /A
	
	return w
End


//Loads and save overlays in dedicated waves Pressure, Time, ...
Function DAVOverlays2009(FrameNr)
	
	Variable FrameNr
	
	Nvar RefRead, t_i, t_0
	Nvar FrameNumber//; FrameNumber=FrameNr
	String/G FrameName, FileHeader, ImageHeader
	
	Variable Temp=0, SV=0, Pressure=0, ElapsedTime=0, i, TempVar
	Wave SampleTempW, PressureW, ElapsedTimeW,StartVoltW
	String Str //FOV="", 
		
	STRUCT UKFileHeader2009 MyFileHeader 
	STRUCT UKImageHeader2009 MyImageHeader
	
	
	
	fsetpos RefRead,0
	FBinRead RefRead, MyFileHeader
	//print MyFileHeader
	
	Fstatus RefRead	
	//Print S_filename
	
	If(StringMatch(S_filename, "*.dat*"))
		// for .dat sequence we have to open the appropriate file
		If(FrameNr!=0)
			Svar DATSeqFilenameList
			Close/a
			Str=StringFromList(FrameNr,DATSeqFilenameList)
			open/R /P=DATSeqPath RefRead as Str
		EndIf
		
		FrameNr=0		
		fsetpos RefRead,104+(0)*(288+1024*1024*2)
		FBinRead RefRead, MyImageHeader
		//print MyImageHeader
		
		// .dat files can have some markups hence the offset of 128 bytes
		If(MyImageHeader.attachedMarkupSize==0)
			fsetpos RefRead, 104+(288+0)*(0+1)+(0)*1024*1024*2
		Else			
			fsetpos RefRead, 104+(288+128)*(0+1)+(0)*1024*1024*2
		EndIf	
	Else	
		fsetpos RefRead,104+(FrameNr)*(288+1024*1024*2)
		FBinRead RefRead, MyImageHeader
		//print MyImageHeader

		fsetpos RefRead, 104+288*(FrameNr+1)+(FrameNr)*1024*1024*2	
	EndIf	
		
	
		
	fstatus RefRead
	
	t_i=0				
	
	t_i=MyImageHeader.imagetime.LONG[0]
	t_i+=2^32*MyImageHeader.imagetime.LONG[1]
		
	If(FrameNr==0)
		t_0=t_i
		ElapsedTimeW[FrameNr]=0
	Else
//		If(t_0==0)			
//			ElapsedTimeW[FrameNr]=0
//		Else
			ElapsedTimeW[FrameNr]=(t_i-t_0)*1e-7 //Elapsed  time (from t_0 to t_i) converted from 100ns to s
//		EndIf
	EndIf
	
	
	
	For(i=0;i<256;i+=1)
		If(MyImageHeader.LEEMdata[i]==39)
			fsetpos RefRead,104+28+i+15+(FrameNr)*(288+1024*1024*2)
			FBinRead/f=4 RefRead, Temp
			//Print "Temp i=", i
			SampleTempW[FrameNr]=Temp
			i+=18
		EndIf
		If(MyImageHeader.LEEMdata[i]==38)
			fsetpos RefRead,104+28+i+16+(FrameNr)*(288+1024*1024*2)
			FBinRead/f=4 RefRead, SV
			//Print "SV i=", i
			StartVoltW[FrameNr]=SV
			i+=19
		EndIf
		
		//106 is the ascii number of the module 'Main'
		If(MyImageHeader.LEEMdata[i]==106)
			//Sometimes the unit 'Torr' (ascii code 'T' is 84) is inserted between the two 'NULL' characters
			//Sometimes there's a space character (ascii code is 32)!!
			If(MyImageHeader.LEEMdata[i+6]==32)
				fsetpos RefRead,104+28+i+8+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, Pressure
				i+=11
			elseif(MyImageHeader.LEEMdata[i+6]==84)
				fsetpos RefRead,104+28+i+11+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, Pressure
				i+=14
			Endif
			PressureW[FrameNr]=Pressure
		EndIf	
	EndFor
End



//----------------------------------------------------------------------------------------------------------------------------------------------------//
//								DAVBuild & DAVExtract &DAVConvert							//
//----------------------------------------------------------------------------------------------------------------------------------------------------//

Function DAVBuild2009() 
	
	Nvar RefRead,NFrames
	Svar DATSeqRootFilename, DATSeqFilenameList
	
	Variable /g RefWrite
	Variable i, Flag
	String TmpStr
	STRUCT UKFileHeader2009 MyFileHeader 
		
	TmpStr=IndexedFile(DATSeqPath,-1,".dav")
	If(StringMatch(TmpStr, "*"+DATSeqRootFilename+"_build.dav*"))
		Deletefile/Z/P=DATSeqPath DATSeqRootFilename+"_build.dav"
	EndIf
	open/P=DATSeqPath RefWrite as DATSeqRootFilename+"_build.dav"
	
	fsetpos RefRead,0
	fsetpos RefWrite,0
	
	FBinRead RefRead, MyFileHeader
	FBinWrite RefWrite, MyFileHeader
	//Print MyFileHeader
	
	Close RefRead
	for(i=0; i<NFrames; i=i+1)
		DAVExtractFrame2009(i,i,"dat")//, RefRead,RefWrite)
		//Print i
	EndFor
	
	Close /A //RefWrite
	open/Z/P=DATSeqPath/R RefRead DATSeqRootFilename+"_build.dav"
End

Function DAVExtract2009(ctrlName) : ButtonControl
	String ctrlName
	
	Nvar RefRead
	
	Nvar FromVar, ToVar, StepVar
	Svar Message
	
	Variable /g RefWrite
	Variable i, NFramesToExtract,Flag,TempVar
	
//	Prompt FromVar, "From frame: "
//	Prompt ToVar,"To:"
//	Prompt StepVar,"Step:"
//	DoPrompt "Extract sequence from a .dav file",FromVar, ToVar, StepVar
	Flag=FromToStepDialog(1)
	
	NFramesToExtract=(ToVar-FromVar)/StepVar+1
	//print NFramesToExtract
	
	If(Flag==0)
		STRUCT UKFileHeader2009 MyFileHeader 
		Fstatus RefRead
		
		open/F="Data Files (*.dav):.dav;All Files:.*;" RefWrite as StringFromList(0, S_fileName,".")+"_extr.dav"
		
		fsetpos RefRead,0
		fsetpos RefWrite,0
		
		FBinRead RefRead, MyFileHeader
		FBinWrite RefWrite, MyFileHeader
		
		for(i=0; i<NFramesToExtract; i+=1)
		
			TempVar=FromVar+i*StepVar	
			Wave 'Wave_i'=DAVLoadFrame2009(TempVar,0)			
			
			DAVWriteFrame2009(NameOfWave('Wave_i'),i)//, RefRead,RefWrite)
			Killwaves/Z 'Wave_i'
			//Print i
		EndFor
		
		//Close RefWrite
	EndIf
End

Function DAVExtractFrame2009(FrameInitialPosition,FrameFinalPosition,OptionStr)//, RefRead,RefWrite)
	Variable FrameInitialPosition,FrameFinalPosition
	String OptionStr
	
	Nvar RefRead,RefWrite

	STRUCT UKImageHeader2009 MyImageHeader
 
	make/D/O/N=(1024,1024) w
	If(stringmatch(OptionStr,"dav"))
//		fsetpos RefRead,104+(FrameInitialPosition)*(288+1024*1024*2)
//		fsetpos RefWrite,104+(FrameFinalPosition)*(288+1024*1024*2)
//		FBinRead RefRead, MyImageHeader
//		FBinWrite RefWrite, MyImageHeader
//		
//		fsetpos RefRead, 104+288*(FrameInitialPosition+1)+(FrameInitialPosition)*1024*1024*2
//		//fsetpos RefRead, 104+288*(1)+(0)*1024*1024*2	
//		fsetpos RefWrite, 104+288*(FrameFinalPosition+1)+(FrameFinalPosition)*1024*1024*2			
//		FBinRead/f=2 RefRead, w
//		FBinWrite/f=2 RefWrite, w
	else
		Svar DATSeqFilenameList

		open/R/Z/P=DATSeqPath RefRead StringFromList(FrameInitialPosition,DATSeqFilenameList)
		fsetpos RefRead,104
		fsetpos RefWrite,104+(FrameFinalPosition)*(288+1024*1024*2)
		FBinRead RefRead, MyImageHeader		
			
		If(MyImageHeader.attachedMarkupSize!=0)
			// .dat files can have some markups hence the offset of 128 bytes
			fsetpos RefRead, 104+(288+128)
			MyImageHeader.attachedMarkupSize=0
		else 
			fsetpos RefRead, 104+(288+0)
		EndIf
		FBinWrite RefWrite, MyImageHeader
			
		//fsetpos RefRead, 104+288*(1)+(0)*1024*1024*2	
		fsetpos RefWrite, 104+288*(FrameFinalPosition+1)+(FrameFinalPosition)*1024*1024*2		
		FBinRead/f=2 RefRead, w
//		Duplicate/O w, $"Frame"+num2str(FrameFinalPosition)
//		newimage/k=1 $"Frame"+num2str(FrameFinalPosition)
//		ModifyImage $"Frame"+num2str(FrameFinalPosition) ctab= {MyImageHeader.ColorScaleLow,MyImageHeader.ColorScaleHigh,Grays}
		
		FBinWrite/f=2 RefWrite, w
		close RefRead
	Endif
	
//	fsetpos RefRead,104+(FromV+FrameNr*StepV)*(288+1024*1024*2)
//	fsetpos RefWrite,104+(FrameNr)*(288+1024*1024*2)
//	FBinRead RefRead, MyImageHeader
//	FBinWrite RefWrite, MyImageHeader
//	
//	fsetpos RefRead, 104+288*(FromV+FrameNr*StepV+1)+(FromV+FrameNr*StepV)*1024*1024*2	
//	fsetpos RefWrite, 104+288*(FrameNr+1)+(FrameNr)*1024*1024*2			
//	FBinRead/f=2 RefRead, w
//	FBinWrite/f=2 RefWrite, w	
	KillWaves w
End

Function DAVWriteFrame2009(FrameName,FramePosition)//, RefRead,RefWrite)
	String FrameName
	Variable FramePosition
	
	Nvar RefRead,RefWrite
	Variable FrameNr=str2num(replacestring("Frame",FrameName,""))
	//Variable FrameNr=str2num(replacestring("Frame",NameOfWave('Wave_i+1'),""))

	STRUCT UKImageHeader2009 MyImageHeader

	Print "Writing : ",FrameName, "in position", FramePosition
	
	// In case DAVWriteFrame is called from DAVBAckground MyImageHeader.ColorScaleHigh and Low have to be modified
	ImageStats /M=1 $FrameName
	//Print V_min, V_max
	MyImageHeader.ColorScaleHigh=V_max
	MyImageHeader.ColorScaleLow=V_min
	//Print MyImageHeader.ColorScaleLow, MyImageHeader.ColorScaleHigh 
	
	fsetpos RefRead,104+(FrameNr)*(288+1024*1024*2)
	fsetpos RefWrite,104+(FramePosition)*(288+1024*1024*2)
	FBinRead RefRead, MyImageHeader
	FBinWrite RefWrite, MyImageHeader

	fsetpos RefWrite, 104+288*(FramePosition+1)+(FramePosition)*1024*1024*2			
	
	FBinWrite/f=2 RefWrite, $FrameName
	
End

Function DAVConvert2009(ctrlName) : ButtonControl
	String ctrlName
	
	Nvar RefRead
	Nvar FromVar, ToVar, StepVar, DurationVar
	Svar FormatStr,OverlayerStr
	Variable i, NFramesToConvert, Flag, FrameRate
	
//	Prompt FromVar, "From frame: "
//	Prompt ToVar,"To:"
//	Prompt StepVar,"Step:"
//	DoPrompt "Convert sequence to .mov format",FromVar, ToVar, StepVar
	Flag=FromToStepDialog(2)
	
	If(Flag==1)
		abort "Convert procedure cancelled"
	Else
	
		NFramesToConvert=trunc((ToVar-FromVar)/StepVar)+1
		FrameRate=NFramesToConvert / DurationVar
		
		If(FrameRate<1)
			 FrameRate=1 
		 Endif
		If(FrameRate>60)
			 FrameRate=60 
		 Endif
		
		If(Stringmatch(FormatStr,".avi"))
			newmovie/A/O/f=(FrameRate)/I
			Print "Converting",NFramesToConvert, "frames to a .avi file at a video rate of", FrameRate,"fr/s"
		Else
			newmovie/L/O/f=(FrameRate)/I
			Print "Converting",NFramesToConvert,"frames to a .mov file at a video rate of",FrameRate,"fr/s"
		EndIf
	
		for(i=0; i<NFramesToConvert; i+=1)
			DAVAddFrame2009(FromVar+i*StepVar,"Yes")
			DoUpdate
			AddMovieFrame
		EndFor
		closemovie
		OverlayerStr="Yes"	
	EndIf
	
End

//----------------------------------------------------------------------------------------------------------------------------------------------------//
//					DAVSlider DAVSliderButton Proc and DAVAdd								//
//----------------------------------------------------------------------------------------------------------------------------------------------------//


Function DAVsliderProc2009(name, value, event) : SliderControl
	String name	// name of this slider control
	Variable value	// value of slider
	Variable event	// bit field: bit 0: value set; 1: mouse down, 
				//   2: mouse up, 3: mouse moved		

	Nvar DoesCursorAExist
	String Str
	
	If(strlen(ImageNameList("", ";"))>0)
		//print "cursor=",CsrInfo(A)
		Str=CsrInfo(A)
		if(strlen(Str)>0)
			DoesCursorAExist=1
			Svar XcursorAStr,YcursorAStr
			XcursorAStr=StringByKey("POINT", Str, ":", ";")
			YcursorAStr=StringByKey("YPOINT", Str, ":", ";")
		Else
			DoesCursorAExist=0
		Endif
	EndIf
	
	//Once the slider pointer is moved and released
	If(event==9)
		//No .dav or .dat frame displayed 
		if(!StringMatch(ImageNameList("", ";"),"*Frame*"))
			DAVLoad2009("DAVLoadButton")
			
		//A .dav or .dat frame is already displayed 
		Else
			DAVAddFrame2009(value,"Yes")	
		EndIf		
		
		If(Exists("M_ROIMask"))
			If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
				Str=StrVarOrDefault("root:TopImageWinName","nop!")
				If(stringmatch(str,"nop!"))
					String/G TopImageWinName
				Else
					Svar TopImageWinName
				EndIf
				TopImageWinName=WinName(0,1,1)
				//RemoveImage M_ROIMask_boundary
				AppendROIBoundary()
				
			Else
				CalculateAndAppendROIBoundary()	
			EndIf
			
			svar TopImageWaveName
			TopImageWaveName=StringFromList(0, ImageNameList("",";"))
			//print TopImageWaveName
			// Imagestats /R=M_ROIMask $TopImageWaveName
			 //print V_npnts*V_avg
			
			
		EndIf		
		
	//		If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
	//	If(event==9)
	//		AppendROIBoundary()
	//	Else
	//		CalculateAndAppendROIBoundary()	
	//	EndIf
	//	print event
	EndIf			
	return 0	// other return values reserved
End

Function DAVSliderButton2009(ctrlName) : ButtonControl
	String ctrlName
	Nvar NFrames, FrameNumber
	String Str
	
	ControlInfo /W=LeemPanel2009 DAVslider
	If(Stringmatch(ctrlName,"*Right*"))
		If(V_Value<NFrames-1)
			V_Value+=1
			DAVAddFrame2009(V_Value, "Yes")
			Slider DAVslider value=V_Value, win=LeemPanel2009
			FrameNumber=V_Value
		EndIf
	Else
		If(V_Value>0)
			V_Value-=1	
			DAVAddFrame2009(V_Value, "Yes")
			Slider DAVslider value=V_Value, win=LeemPanel2009
			FrameNumber=V_Value
		EndIf
	Endif
	
	If(Exists("M_ROIMask"))
		If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
			Str=StrVarOrDefault("root:TopImageWinName","nop!")
			If(stringmatch(str,"nop!"))
				String/G TopImageWinName
			Else
				Svar TopImageWinName
			EndIf
			TopImageWinName=WinName(0,1,1)
			//RemoveImage M_ROIMask_boundary
			AppendROIBoundary()
			
		Else
			CalculateAndAppendROIBoundary()	
		EndIf
		
		svar TopImageWaveName
		TopImageWaveName=StringFromList(0, ImageNameList("",";"))
		//print TopImageWaveName
		// Imagestats /R=M_ROIMask $TopImageWaveName
		 //print V_npnts*V_avg
	EndIf

End


Function/WAVE DAVAddFrame2009(FrameNr, Rmve)
	Variable FrameNr
	String Rmve
	
	Svar OverlayerStr
	String/G TopImageWaveName=""
	String Str
		
	Str=StrVarOrDefault("root:TopImageWinName","nop!")
	If(stringmatch(str,"nop!"))
		String/G TopImageWinName
	Else
		Svar TopImageWinName
	EndIf
	TopImageWinName=WinName(0,1,1)

	If(stringmatch(Rmve,"Yes"))
		TopImageWaveName=StringFromList(0, ImageNameList("",";"))
		RemoveImage $TopImageWaveName
		KillWaves/Z $TopImageWaveName
		
		If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
			RemoveImage M_ROIMask_boundary
		EndIf
	EndIf
	
	Nvar FrameNumber; FrameNumber=FrameNr
	Nvar RefRead, t_i, t_0
	Svar FrameName, FileHeader, ImageHeader
	
	Variable Temp=0, SV=0, Pressure=0, ElapsedTime=0,i,Obj=0
	String FOV=""
		
	STRUCT UKFileHeader2009 MyFileHeader 
	STRUCT UKImageHeader2009 MyImageHeader
	
	Fstatus RefRead
	// for .dat sequence we have to open the appropriate file
	If(StringMatch(S_filename, "*.dat*"))//&&FrameNr!=0)
		Svar DATSeqFilenameList
		Nvar NFrames
		Close/a
		Str=StringFromList(FrameNr,DATSeqFilenameList)
		open/R /P=DATSeqPath RefRead as Str
		FrameNr=0
	EndIf
	t_i=0	
	fsetpos RefRead,0
	FBinRead RefRead, MyFileHeader
	//print MyFileHeader.attachedRecipeSize
	
	fsetpos RefRead,104+(FrameNr)*(288+1024*1024*2)
	FBinRead RefRead, MyImageHeader
	//print MyImageHeader
			
	
	t_i=MyImageHeader.imagetime.LONG[0]
	t_i+=2^32*MyImageHeader.imagetime.LONG[1]
		
	If(FrameNr==0)
		t_0=t_i
	Else
		If(t_0==0)			
			ElapsedTime=0
		Else
			ElapsedTime=(t_i-t_0)*1e-7 //Elapsed  time (from t_0 to t_i) converted from 100ns to s
		EndIf
	EndIf
	
	t_i*=1e-7 // t_i converted from 100ns to s
	t_i-=9561628800 // t_i converted from Windows Filetime format (01/01/1601) to Mac Epoch format (01/01/1970)
	
	For(i=0;i<256;i+=1)
		If(MyImageHeader.LEEMdata[i]==39)
			fsetpos RefRead,104+28+i+15+(FrameNr)*(288+1024*1024*2)
			FBinRead/f=4 RefRead, Temp
			//Print "Temp i=", i
			i+=18
		EndIf
		If(MyImageHeader.LEEMdata[i]==38)
			fsetpos RefRead,104+28+i+16+(FrameNr)*(288+1024*1024*2)
			FBinRead/f=4 RefRead, SV
			//Print "SV i=", i
			i+=19
		EndIf
		//110 is the ascii number of the module FOV but it's also the character 'n'
		If(MyImageHeader.LEEMdata[i]==110)
			//To make sure we are not in the 'Emission Cur.' block or in the 'Main'
			If(MyImageHeader.LEEMdata[i+1]!=32 && MyImageHeader.LEEMdata[i+1]!=0)
				//FOV=LEED
				If(MyImageHeader.LEEMdata[i+1]==76)
					FOV="LEED"
					// To reach the end-1 of this block
					i+=9
				Else
					FOV=num2char(MyImageHeader.LEEMdata[i+1])
					FOV+=num2char(MyImageHeader.LEEMdata[i+2])
					FOV+=num2char(MyImageHeader.LEEMdata[i+3])
					FOV+=num2char(MyImageHeader.LEEMdata[i+4])
					//Print "FOV i=",i
					//print "LEEM", MyImageHeader.LEEMdata[i+1]
											
					//3 digit FOV:
					If(MyImageHeader.LEEMdata[i+6]==42)	
						FOV+=num2char(MyImageHeader.LEEMdata[i+5])
						//Print "FOV i=",i
						//print "LEEM", MyImageHeader.LEEMdata[i+1]
						i+=1
					EndIf
					i+=10
				EndIf	
			EndIf
		EndIf
		//106 is the ascii number of the module 'Main'
		If(MyImageHeader.LEEMdata[i]==106)
			//Sometimes the unit 'Torr' (ascii code 'T' is 84) is inserted between the two 'NULL' characters
			//Sometimes there's a space character (ascii code is 32)!!
			If(MyImageHeader.LEEMdata[i+6]==32)
				fsetpos RefRead,104+28+i+8+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, Pressure
				i+=11
			elseif(MyImageHeader.LEEMdata[i+6]==84)
				fsetpos RefRead,104+28+i+11+(FrameNr)*(288+1024*1024*2)
				FBinRead/f=4 RefRead, Pressure
				i+=14
			Endif
		EndIf	
			
		//11 is the ascii number of the module Obj
		If(MyImageHeader.LEEMdata[i]==11)
			fsetpos RefRead,104+28+i+12+(FrameNr)*(288+1024*1024*2)
			FBinRead/f=4 RefRead, Obj
			i+=5
		EndIf
	EndFor
//	fsetpos RefRead,104+43+(FrameNr)*(288+1024*1024*2)
//	FBinRead/f=4 RefRead, Temp
//	fsetpos RefRead,104+63+(FrameNr)*(288+1024*1024*2)
//	FBinRead/f=4 RefRead, SV
//	FOV=num2char(MyImageHeader.LEEMdata[40])
//	FOV+=num2char(MyImageHeader.LEEMdata[41])
//	fsetpos RefRead,104+89+(FrameNr)*(288+1024*1024*2)
//	FBinRead/f=4 RefRead, Pressure
//	Str=num2str(MyImageHeader.LEEMdata[0])
//	For(i=1;i<256;i+=1)
//		Str+=" "
//		Str+=num2str(MyImageHeader.LEEMdata[i])
//		//Print MyImageHeader.LEEMdata[i]
//	EndFor
//	Print Str	
	
	FrameName="Frame"+num2str(FrameNr)
	make/D/O/N=(1024,1024) $FrameName /WAVE=w
	If(MyImageHeader.attachedMarkupSize==0)
		// .dav files have no markup
		fsetpos RefRead, 104+288*(FrameNr+1)+(FrameNr)*1024*1024*2
	Else
		// .dat files can have some markups hence the offset of 128 bytes
		fsetpos RefRead, 104+(288+128)*(0+1)+(0)*1024*1024*2
	EndIf
	FBinRead/f=2 RefRead, w
	
	//duplicate/O w, $FrameName
	//KillWaves w
	fstatus RefRead
	
	AppendImage/T/W=$TopImageWinName $FrameName
	ModifyImage $FrameName ctab= {*,*,Grays},log=1
	//ModifyImage $FrameName ctab= {MyImageHeader.ColorScaleLow,MyImageHeader.ColorScaleHigh,Grays}
	//Print MyImageHeader.ColorScaleLow, MyImageHeader.ColorScaleHigh
	
	//AppendImage $FrameName
	SetAxis/A/R left;
	
	// Add overlayer data : sample temperature, FOV, SV, ...
	If(stringmatch(OverlayerStr,"No"))
		TextBox/C/N=Title/V=0
		TextBox/C/N=Day/V=0
		TextBox/C/N=Ell/V=0
		TextBox/C/N=SV/V=0
		TextBox/C/N=Temp/V=0
		TextBox/C/N=Press/V=0
		TextBox/C/N=FOV/V=0
		TextBox/C/N=Obj/V=0
	Else
		If(StringMatch(S_fileName,"*.dav*"))
			TextBox/C/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName+" "+FrameName
		Else
			TextBox/C/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName
		EndIf
		TextBox/N=Day/V=1/C"\\Z10\\K(65280,65280,0)"+secs2date(t_i,0)+" "+secs2time(t_i,1)
		TextBox/N=Ell/V=1/C"\\Z10\\K(65280,65280,0)Elapsed time="+num2str(ElapsedTime)+"s"
		TextBox/N=Temp/V=1/C"\\Z10\\K(65280,65280,0) T\Bsample\M\\Z10="+num2str(Temp)+"C"
		TextBox/N=Press/V=1/C"\\Z10\\K(65280,65280,0) P="+num2str(Pressure)+"Torr"
		TextBox/N=FOV/F=0/B=1/A=LT/X=79/Y=93/C"\\Z10\\K(65280,65280,0) FOV="+FOV//+"m"
		TextBox/N=SV/F=0/B=1/A=LT/X=79/Y=90/C"\\Z10\\K(65280,65280,0) SV="+num2str(SV)+"V"
		TextBox/N=Obj/F=0/B=1/A=LT/X=79/Y=96/C"\\Z10\\K(65280,65280,0) Obj.="+num2str(Obj)+"mA"
	EndIf
	
	//Adjust Image Contrast if "WMContrastAdjustGraph" Window is loaded 
	Str=WinList("*",";","WIN:1")
	if(StringMatch(Str,"*WMContrastAdjustGraph*"))
		SVAR ImGrfName=root:Packages:WMImProcess:ImageContrast:ImGrfName
		ImGrfName=WinName(0,1,1)
		WMContrastButtonProc("Apply")	
				 
		//Kill *_CLU waves
		variable b=0
		do
			Str= StringFromList(b,WaveList("*_CLU",",",""),",")
			if( strlen(Str) == 0 )
				break
			endif
			Wave w=$Str
			//print nameofwave(w)
			KillWaves/Z w
			b += 1
		while (1)	// exit is via break statement		
	EndIf
	
	//Add cursor A if it was displayed before calling DAVAddFrame function
	Nvar DoesCursorAExist
	If(DoesCursorAExist==1)
		Svar XcursorAStr,YcursorAStr,TopImageWinName,TopImageWaveName
		TopImageWaveName=StringFromList(0, ImageNameList("",";"))
		Wave XPos,YPos, SampleWave
		WaveStats/Q/Z/M=1 SampleWave
		//Print TopImageWinName,TopImageWaveName,V_npnts-1,XPos[V_npnts-1],YPos[V_npnts-1]
		//cursor/C=(65535,0,0)/I/H=1/W=$TopImageWinName A, $TopImageWaveName,XPos[V_npnts-1],YPos[V_npnts-1]
		cursor/C=(65535,0,0)/I/H=1/W=$TopImageWinName A, $TopImageWaveName,str2num(XcursorAStr),str2num(YcursorAStr)
	EndIf
	
	return w
End
	
//----------------------------------------------------------------------------------------------------------------------------------------------------------//
//										Loads .mov Movies										//
//----------------------------------------------------------------------------------------------------------------------------------------------------------//


//Function LoadMovie(B_Struct) : ButtonControl
//	STRUCT WMButtonAction &B_Struct
	//print B_Struct.eventCode
	//print B_Struct.ctrlName
	
//Exists in Analysis_LEEM2015
//Function LoadMOV(ctrlName) : ButtonControl
//	String ctrlName
//	
//	Variable/G FrameIndex=0
//	//Variable V_flag
//	
//	PlayMovie/W=(1,1,201,201)/Z
//	//print V_flag
//	If(!V_flag)
//		Nvar MovFileLoaded
//		MovFileLoaded=1
//		PlayMovieAction gotoBeginning, extract
//		
//		If(WaveExists(M_MovieFrame))
//			ImageTransform rgb2gray M_MovieFrame
//			duplicate/O M_RGB2Gray, Frame0
//			KillWaves M_RGB2Gray
//		Endif
//		
//		DoWindow/K Frame
//		//AppendImage/W=Frame Frame0
//		NewImage  /N=Frame /G=1 /K=1 /S=0 Frame0
//		showinfo
//		//ModifyGraph width=200*dimsize(Frame0,0)/dimsize(Frame0,1),height=200//width=719,height=683
//		AutoPositionWindow/E/M=0
//		
//		TextBox/N=TextBoxFrameName/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)Frame0"
//	EndIf
//End

//----------------------------------------------------------------------------------------------------------------------------------------------------//
//								 Dialog function												//
//----------------------------------------------------------------------------------------------------------------------------------------------------/
//Exists in LEEM_Analysis2015
//Function FromToStepDialog(a)
//	Variable a
//	
//	Nvar FromVar, ToVar, StepVar, DurationVar,PolyOrderVar
//	Svar YesNoStr, OptionStr, BlackStr,ManualStr, FormatStr, OverlayerStr
//	
//	Variable FromV=0
//	Variable ToV=10
//	Variable StepV=1, Thresh, DurationV=10,PolyOrderV=2
//	String Mess,YesNoS, OptionS="Nothing", BlackS, ManualS, FormatS,OverlayerS
//	
//	Prompt FromV, "From frame: "
//	Prompt ToV,"To:"
//	Prompt StepV,"Step:"
//	
//	switch(a)												// numeric switch
//		case 1:
//			Mess="Extract sequence from a .dav file"
//			DoPrompt Mess,FromV, ToV, StepV
//			FromVar=FromV;ToVar=ToV;StepVar=StepV
//			break
//			
//		case 2:
//			Mess="Convert sequence to .mov or .avi format"
//			Prompt FormatS,"Movie file Format ?", popup ".avi;.mov"
//			Prompt DurationV,"Movie Duration (s) ?"
//			Prompt OverlayerS,"Include overlayer data ?", popup "Yes;No"
//			DoPrompt/HELP="Frame rate =[# of frames to convert]/[Movie duration]" Mess,FromV, ToV, StepV,FormatS,DurationV,OverlayerS
//			FromVar=FromV;ToVar=ToV;StepVar=StepV
//			FormatStr=FormatS; DurationVar = DurationV;OverlayerStr=OverlayerS
//			break
//			
//		case 3:
//			Mess="Drift Correction"
//			Prompt ManualS,"Manual Drift Correction ?", popup "Yes;No"
//			DoPrompt Mess,FromV, ToV, StepV,ManualS
//			If(stringmatch(ManualS,"No"))
//				Prompt YesNoS,"Use ROI mask ?", popup "Yes;No"
//				DoPrompt Mess,YesNoS
//				YesNoStr=YesNoS
//			EndIf
//			FromVar=FromV;ToVar=ToV;StepVar=StepV;ManualStr=ManualS
//			break
//			
//		case 4:
//			Mess="Particle Analysis"
//			Prompt YesNoS,"Use ROI mask ?", popup "Yes;No"
//			Prompt OptionS,"Thresholding Method ?", popup "Manual;Iterative"
//			Prompt BlackS,"Black Particles ?", popup "Yes;No"
//			DoPrompt Mess,FromV, ToV, StepV,YesNoS,OptionS, BlackS
//			FromVar=FromV;ToVar=ToV;StepVar=StepV;YesNoStr=YesNoS
//			OptionStr=OptionS; BlackStr=BlackS
//			break
//			
//		case 5:
//			Mess="ROI averaged Intensity"
//			DoPrompt/HELP="Evolution of the average intensity of a region of interest (ROI)" Mess,FromV, ToV, StepV
//			FromVar=FromV;ToVar=ToV;StepVar=StepV
//			break
//			
//		case 6:
//			Mess="Remove Background"
//			Prompt OptionS,"Reference Image (M_RemovedBackground) ?", popup "No;Yes"
//			Prompt PolyOrderV, "Background polynomial Order"
//			DoPrompt/HELP="Removes a Polynomial background determine from a region of interest (ROI) or a Reference image to be named M_RemovedBackground" Mess,FromV, ToV, StepV,OptionS,PolyOrderV
//			OptionStr=OptionS,FromVar=FromV;ToVar=ToV;StepVar=StepV;PolyOrderVar=PolyOrderV
//			break
//			
//		case 7:
//			Mess="ROI Workfunction Measurements"
//			DoPrompt/HELP="Workfunction 2D Map of a region of interest (ROI)" Mess,FromV, ToV, StepV
//			FromVar=FromV;ToVar=ToV;StepVar=StepV
//			break						
//	endswitch
//	
//	If(stringmatch(OptionS,"Manual"))
//		Variable/G Threshold
//		
//		Prompt Thresh,"Threshold value ?"
//		DoPrompt "Particle Analysis", Thresh
//		Threshold=Thresh
//	EndIf
//	
//	Return V_flag
//End

//----------------------------------------------------------------------------------------------------------------------------------------------------------//
//										Loads Single Pictures										//
//----------------------------------------------------------------------------------------------------------------------------------------------------------//

Function Load2009(ctrlName) : ButtonControl
	String ctrlName
	
	String/G FileName
	String/G TopImageWinName
	Svar TopImageWaveName
	
	//newpath/Q/O MyPath, "C:Users:Fabien:Documents:Recherche:Dmouillage:LEEM"
	
	ControlInfo ImageLoadPopup
		
	if(stringmatch(S_value,"*dat*"))
		Variable/G RefRead=0,NFrames=0
		
		open/R /F="Data Files (*.dat):.dat;All Files:.*;" RefRead
		fstatus RefRead
		If(V_flag==0)
			print "Open procedure failed"
		else	
			Print "Loading Image "+S_path+S_fileName
			DAVLoadFrame2009(0,1)			
		EndIf			
	Else
		ImageLoad /O /T=$S_value//P=PathToEchantillons
		If(V_flag)
			FileName=S_fileName
			TopImageWaveName=StringFromList(0,S_WaveNames) // i.e. Wave0
			NewImage  /G=1 /K=1 /S=0 $TopImageWaveName
			TopImageWinName=WinName(0,1,1) // i.e. Graph0
			showinfo
			//ModifyGraph width=500*dimsize($TopImageWaveName,0)/dimsize($TopImageWaveName,1),height=500//width=719,height=683
			TextBox/C/N=TextBoxFileName/A=MC "\\Z11\K(0,0,65280)"+FileName
			TextBox/C/N=TextBoxFileName/X=-32.99/Y=45.60
		EndIf
	EndIf
End
	
		