//----------------------------------------------------------------------------------------------------------------------------------------------------------//
//									LEEM Image Analysis											//
//																								// 
//										F. Cheynis												//
//																								//
//										31/01/2019												//
//----------------------------------------------------------------------------------------------------------------------------------------------------------//

// 2019-01-31 Version: Debug LEEM data block and LEEM data array issue
// 2017-06-26 Version:	- DAVSliderButton function includes ROI boundaries when a ROI mask is used
//					- DAVWriteFrame has been modified (CallRefName string added). In some cases
//					the 'write-a-frame' procedure needs first to read it and sometimes (drift correction)
//					the frame has been already read. In other words it's either a 'read-and-write' or a
//					'simply-write' procedure. 
// 2016-09-02 Version:	Elmitec file format changes have been implemented for LoadFrame, WriteFrame
// 2016-06-08 Version:	1st Elmitec file format change implementation 
// 2016-05-03 Version:	GUI updated and left/right browsing buttons added
// 2016-03-29 Version:	Killwaves added in Extract procedure & Overlayers parsing updated including 
//					Obj. lens, Pressure and FOV
// 2014-05-23 Version:	MyImageHeader.ColorScaleLow and MyImageHeader.ColorScaleHigh can be
//					modified by Background corrections. This is implementd in DAVWriteFrame. 
// 2012-10-15 Version: Remove Backrground implemented
// 2011-01-24 Version: Overlayer data can be included or not in the "convert" procedure
// 2010-11-24 Version: dat Sequences 
// 2010-01-29 Version: Drift correction pre-scan


#include "LEEM_Particle Extensions2015"
#include "LEEM_ROI Extensions2015"
#include "LEEM_Drift Correction2015"
#include "LEEM_Remove Background2015"
//#include "aaa_MovieMakers"

#pragma rtGlobals=1		// Use modern global access method.

Menu "Macros"
	"LEEM Analysis 2015", LeemPanel2015()
	"-",
End

Window LeemPanel2015() : Panel

	Variable /g  FrameNumber=0,t_i=0,t_0=0,MovFileLoaded=0
	Variable /g FromVar, ToVar, StepVar, DurationVar, DoesCursorAExist,PolyOrderVar
	Variable /g FileHeaderSize=0,ImageHeaderSize=0, LEEMDataBlockSize=0
	String/G YesNoStr,OptionStr, BlackStr, ManualStr,FormatStr,OverlayerStr,XcursorAStr,YcursorAStr
	
	//newpath/Q/O MyPath, "C:Users:Fabien:Documents:Recherche:Dmouillage:LEEM"
	PauseUpdate; Silent 1		// building window...
	NewPanel/K=1 /W=(696,62,1032,337) as "LEEM 2015"
	ModifyPanel cbRGB=(56576,56576,56576)
	SetDrawLayer UserBack
	SetDrawEnv fsize= 11,textrgb= (0,15872,65280)
	DrawText 158,275,"Rev. Sci. Instrum., 85 (2014) 043705"
	GroupBox FrameGroupBox,pos={6,208},size={322,51},title="\\K(0,15872,65280)Single Frames"
	GroupBox FrameGroupBox,labelBack=(48896,52992,65280),font="Arial",frame=0
	GroupBox MOVGroupBox,pos={6,147},size={322,51},title="\\F'Arial'\\K(0,15872,65280).mov Files"
	GroupBox MOVGroupBox,labelBack=(48896,52992,65280),frame=0
	GroupBox DAVGroupBox,pos={6,1},size={322,139},title="\\K(0,15872,65280).dav Files and .dat Sequences"
	GroupBox DAVGroupBox,labelBack=(48896,52992,65280),font="Arial",frame=0
	Button DAVOpenButton,pos={19,27},size={65,22},proc=SeqOpen,title="Open .dav",font="Arial"
	Button DATSeqOpenButton,pos={180,27},size={65,22},proc=SeqOpen,title="Open Seq",font="Arial"
	Button DAVLoadButton,pos={265,21},size={60,35},proc=DAVLoad,title="(re)Load\rFrame",font="Arial"
	SetVariable DAVSetvar,pos={116,30},size={30,16},title=" "
	SetVariable DAVSetvar,help={"Index of the frame to be loaded (>=0)"}
	SetVariable DAVSetvar,limits={0,200,0},value= FrameNumber,styledText= 1
	Slider DAVslider,pos={28,58},size={218,52},proc=DAVsliderProc
	Slider DAVslider,limits={0,166,1},variable= FrameNumber,vert= 0,ticks= 10
	Button LoadImageButton,pos={82,232},size={79,22},proc=LoadSingleImage,title="Load Image",font="Arial"
	PopupMenu ImageLoadPopup,pos={199,232},size={47,21},title="\\F'Arial'"
	PopupMenu ImageLoadPopup,mode=3,popvalue="dat",value= #"\"dat;tiff;png;jpeg\""
	Button LoadMOVButton,pos={128,168},size={79,22},proc=LoadMov,title="Load .mov",font="Arial"
	Button DAVExtractButton,pos={265,62},size={60,20},proc=DAVExtract,title="Extract",font="Arial"
	Button DAVConvertButton,pos={265,88},size={60,20},proc=DAVConvert,title="Convert",font="Arial"
	Button DAVDriftButton,pos={102,115},size={60,20},proc=DAVDrift,title="Drift Corr.",font="Arial"
	Button DAVPartButton,pos={183,115},size={65,20},proc=DAVPart,title="Part. Anal.",font="Arial"
	Button DAVROIButton,pos={265,115},size={60,20},proc=DAVROI,title="I\\B ROI",font="Arial"
	Button DAVBackgroundButton,pos={21,115},size={60,20},proc=DAVBackground,title="Backgrnd",font="Arial"
	Button DAVSliderButtonRight,pos={245,56},size={19,20},title="\\W649",proc=DAVSliderButton
	Button DAVSliderButtonLeft,pos={9,57},size={20,19},title="\\W646",proc=DAVSliderButton
EndMacro



//----------------------------------------------------------------------------------------------------------------------------------------------------//
//								DAV Files and DAT Sequences								//
//----------------------------------------------------------------------------------------------------------------------------------------------------//

Structure LONGLONG
	uint32 LONG[2]
EndStructure


Structure UKFileHeader
	char id[20]
	int16 size
	int16 version
	int16 BitsPerPixel
	int16 CameraBitsPerPixel
	int16 MCPDiameterInPixels
	uchar hBinning, vBinning
	STRUCT LONGLONG spare			
	int16 ImageWidth, ImageHeight
	int16 NrImages
	int16 attachedRecipeSize
	uchar filler[56]	  
EndStructure

Structure UKImageHeader
	int16 size
	int16 version
	int16 ColorScaleLow,ColorScaleHigh
	STRUCT LONGLONG imagetime
	int16 MaskXshift, MaskYshift
	uint16 RotateMask
	int16 attachedMarkupSize
	int16 spin
	int16 LEEMdataVersion
	uchar LEEMdata[240]
	uchar applied_processing
	uchar gray_adjust_zone
	uint16 backgroundvalue
	uchar desired_rendering
	uchar spare
	int16 rendering_argShort
	float rendering_argFloat
	int16 desired_rotation
	int16 rotation_offset
	int16 filler[2]
EndStructure

Structure DummyStruct
	float Float
EndStructure


//Dialog to open a .dat file sequence or a .dav file and Generation of the first image
Function SeqOpen(ctrlName) : ButtonControl
	String ctrlName
	
	Variable/G RefRead=0,NFrames=0
	String/G DATSeqRootFilename,DATSeqFilenameList
	
	Nvar FrameNumber; FrameNumber=0
	Nvar FileHeaderSize, ImageHeaderSize

	
	//newpath/Q/O MyPath, "V:demouillage:SOI_0per:SOI-0per-LEEM-10"
	String ExtractOverlaysStr, Str,TmpStr
	
	close/a
	Setdatafolder root:
	If(StringMatch(ctrlName,"*DAV*"))
		//open/R/P=MyPath /F="Data Files (*.dav):.dav;All Files:.*;" RefRead
		open/R /F="Data Files (*.dav):.dav;All Files:.*;" RefRead		
	Else
		//open/R/P=MyPath /F="Data Files (*.dat):.dat;All Files:.*;" RefRead
		open/R /F="Data Files (*.dat):.dat;All Files:.*;" RefRead
	EndIf
	
	fstatus RefRead
	If(V_flag==0)
		abort "Open procedure failed"
	Else
		Print " Loading Sequence "+S_path+S_fileName
	Endif
	
	Str=StrVarOrDefault("root:TopImageWinName","nop!")
	If(stringmatch(str,"nop!"))
		String/G TopImageWinName
	Else
		Svar TopImageWinName
	EndIf
	TopImageWinName=WinName(0,1,1)
	
//	If((strlen(TopImageWinName)!=0)&&(!stringmatch(TopImageWinName,"*WMContrastAdjustGraph*")))
//		KillWindow $TopImageWinName
//	Endif

	Nvar MovFileLoaded
	MovFileLoaded=0
	If(StringMatch(ctrlName,"*DAV*"))
		DAVLoadImageHeaderSizes()
		//NFrames=floor((V_logEOF-FileHeaderSize)/(ImageHeaderSize+1024*1024*2))
		DATSeqFilenameList=""
	Else
		String BuildADavS
		Variable i, NDATFiles
		
		Killwaves /Z ImageHeaderSizeW
		NewPath/Q/Z/O DATSeqPath, S_path
		
		//Removes "_nnnn.dat" or "-nnnn.dat" or "nnn.dat" to the name of the opened file to define  DATSeqRootFilename
		TmpStr=RemoveEnding(S_fileName, ".dat")
		SplitString/E="((\\x2D|\\x5F)+[[:digit:]]+|([[:digit:]]+))$" TmpStr, Str
		DATSeqRootFilename=RemoveEnding(TmpStr, Str)
		
		TmpStr=IndexedFile(DATSeqPath, -1, ".dat")
		NDATFiles=ItemsInList(TmpStr)
		
		//Removes .dat files from the list that do not share a common filename root with the opened file		
		for(i=0;i<NDATFiles; i+=1)
			if(!StringMatch(StringFromList(i,TmpStr),"*"+DATSeqRootFilename+"*"))
				TmpStr=RemoveListItem(i,TmpStr)
				i-=1
				NDATFiles-=1
			EndIf
		EndFor
	
		DATSeqFilenameList=TmpStr
		NFrames=NDATFiles
		
		If(NFrames>1)
			Prompt BuildADavS,"Do you want to build a .dav file from the current .dat sequence?", popup "Yes;No"
			DoPrompt "",BuildADavS
			if((V_flag==0)&&(stringmatch(BuildADavS,"Yes"))	)		
				DAVBuild()
				DAVLoadImageHeaderSizes()
			Endif
		Endif
		//DAVLoadFrame(WhichListItem(S_filename, DATSeqFilenameList,";"),2)
	EndIf
	
	DAVLoadFrame(0,2)
	
	If(NFrames==1)
		Slider DAVslider, disable=2, win=LeemPanel2015
		Button DAVSliderButtonRight, disable=2, win=LeemPanel2015
		Button DAVSliderButtonLeft, disable=2, win=LeemPanel2015
		//SetVariable DAVSetvar, noedit=1, value=(NFrames-1), win=LeemPanel2015
	Else
		SetVariable DAVSetvar, limits={0,NFrames-1,0}, noedit=0,win=LeemPanel2015
		Slider DAVslider, limits={0,NFrames-1,1} ,disable=0,win=LeemPanel2015
		Button DAVSliderButtonRight, disable=0, win=LeemPanel2015
		Button DAVSliderButtonLeft, disable=0, win=LeemPanel2015
	Endif
	
	//Clean potential ROIMask waves from previous sequence
	If(Exists("M_ROIMask"))
		KillWaves/Z M_ROIMask
	EndIf
	If(Exists("M_ROIMask_boundary"))
		KillWaves/Z M_ROIMask_boundary
	EndIf
	
	
	WMCreateImageROIPanel()
	Nvar DoesCursorAExist;DoesCursorAExist=0 //Initialise the flag telling if cursor A is on the graph
	
	//Extract data overlays in dedicated waves Temperature, Pressure, Time, ...
	Prompt ExtractOverlaysStr,"Do you want to extract the sequence overlays?", popup "No;Yes"
	DoPrompt "", ExtractOverlaysStr
	if(V_flag==0)			
		if(stringmatch(ExtractOverlaysStr,"*Yes*"))
			make/O/N=(NFrames) SampleTempW, PressureW, ElapsedTimeW,StartVoltW
			SampleTempW=nan; PressureW=nan; ElapsedTimeW=nan; StartVoltW=nan
			For(i=0;i<NFrames; i+=1)
				DAVOverlays(i)
			Endfor			
		Endif
	Endif
End

//(re)Load a Frame from a .dav file or a .dat sequence
Function DAVLoad(ctrlName) : ButtonControl
	String ctrlName
	Nvar FrameNumber, RefRead 
	
	String Str
	
	Str=StrVarOrDefault("root:TopImageWinName","nop!")
	If(stringmatch(str,"nop!"))
		String/G TopImageWinName
	Else
		Svar TopImageWinName
	EndIf
	TopImageWinName=WinName(0,1,1)
	If(strlen(TopImageWinName)!=0)
		KillWindow $TopImageWinName
	Endif
	
	Fstatus RefRead
	If(V_flag!=0)
		// Opens the appropriate .dat image of a sequence
		If(StringMatch(S_filename, "*.dat*"))
			Svar DATSeqFilenameList
			If(strlen(DATSeqFilenameList)!=0)
				Close/a
				Str=StringFromList(FrameNumber,DATSeqFilenameList)
				open/R /P=DATSeqPath RefRead as Str
			EndIf
		EndIf	
		DAVLoadFrame(FrameNumber,1)
	EndIf	
End

Function/WAVE DAVLoadFrame(FrameNr,DisplayMode)
	
	Variable FrameNr, DisplayMode //DisplayMode=0: no display, 1: unless needed add image to an existing graph
											   //2: force to create a new window (for newly opened files)
	
	Nvar RefRead, t_i, t_0
	Nvar FrameNumber//; FrameNumber=FrameNr
	Nvar FileHeaderSize, ImageHeaderSize,LEEMDataBlockSize
	Svar OverlayerStr

	String/G FrameName//, FileHeader, ImageHeader
	
	Variable Temp=0, SV=0, Pressure=0, ElapsedTime=0, i=0,imax=0, TempVar,Obj=0
	Variable TempLoaded=0, SVLoaded=0, PressureLoaded=0, ObjLoaded=0,FOVLoaded=0
	String FOV="", Str
		
	STRUCT UKFileHeader MyFileHeader 
	STRUCT UKImageHeader MyImageHeader
	STRUCT DummyStruct MyDummyStruct
	
	TempVar=FrameNr
			
	Fstatus RefRead
	//Print S_filename
	If(StringMatch(S_filename, "*.dat*"))
		Str=StrVarOrDefault("root:DATSeqFilenameList","nop!")
		If(stringmatch(Str,"nop!"))
			String/G DATSeqFilenameList
		Else
			Svar DATSeqFilenameList
		EndIf
		// For .dat image sequence we have to open the appropriate file
		If(strlen(DATSeqFilenameList)!=0)
			//i=WhichListItem(S_filename, DATSeqFilenameList,";")
			Slider DAVslider value=FrameNr, win=LeemPanel2015
			FrameNumber=FrameNr			
			Close RefRead
			Str=StringFromList(FrameNr,DATSeqFilenameList)
			open/R /P=DATSeqPath RefRead as Str
			FrameNr=0
			//Print Str
		EndIf
	Endif
	
	//For .dav files and single .dat files the appropriate file is already opened from SeqOpen or LoadSingleImage
	fsetpos RefRead,0
	FBinRead RefRead, MyFileHeader
	// File header: 104 bytes and potentially 128 bytes if attachedRecipeSize>0
	if(MyFileHeader.attachedRecipeSize>0)		
		FileHeaderSize=104+128
	else
		FileHeaderSize=104
	Endif
		
	//Only applies to the 1st image of .dav files and .dat files
	If(FrameNr==0)
		fsetpos RefRead,FileHeaderSize
		FBinRead RefRead, MyImageHeader
		if(MyImageHeader.LEEMDataVersion>2)
			LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		Else
			LEEMDataBlockSize=0
		Endif
		//Elmitec files can have additional markup blocks hence the offsets of 128 bytes
		If(MyImageHeader.attachedMarkupSize==0)
			//no markups
			ImageHeaderSize= 288+LEEMDataBlockSize
		Else		
			//Markup blocks multiple of 128 bytes after image header	
			ImageHeaderSize= 288+128*((trunc(MyImageHeader.attachedMarkupSize/128))+1)+LEEMDataBlockSize
		EndIf
		fsetpos RefRead, FileHeaderSize+ImageHeaderSize
	//Only applies to .dav files as FrameNr=0 for .dat files
	Else
		wave ImageHeaderSizeW
		Wavestats/Q/Z/M=1/R=[0,FrameNr-1] ImageHeaderSizeW
		fsetpos RefRead,FileHeaderSize+(FrameNr)*(1024*1024*2)+V_sum	
		FBinRead RefRead, MyImageHeader
		if(MyImageHeader.LEEMDataVersion>2)
			LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		Else
			LEEMDataBlockSize=0
		Endif
		fsetpos RefRead, FileHeaderSize+(FrameNr)*1024*1024*2+V_sum+ImageHeaderSizeW[FrameNr]
	EndIf
			
	FrameName="Frame"+num2str(FrameNr)
	make/D/O/N=(1024,1024) $FrameName /WAVE=w	
	FBinRead/f=2 RefRead, w
		
	fstatus RefRead
	If(DisplayMode!=0)
		t_i=MyImageHeader.imagetime.LONG[0]
		t_i+=2^32*MyImageHeader.imagetime.LONG[1]
		If(TempVar==0)
//			t_i=MyImageHeader.imagetime.LONG[0]
//			t_i+=2^32*MyImageHeader.imagetime.LONG[1]
			t_0=t_i
		Else
//			fsetpos RefRead, FileHeaderSize+(FrameNr)*(1024*1024*2+ImageHeaderSize)
//			FBinRead RefRead, MyImageHeader
//			t_i=MyImageHeader.imagetime.LONG[0]
//			t_i+=2^32*MyImageHeader.imagetime.LONG[1]
			If(t_0==0)			
				ElapsedTime=0
			Else
				ElapsedTime=(t_i-t_0)*1e-7 //Elapsed  time (from t_0 to t_i) converted from 100ns to s
			EndIf
		EndIf
		
		t_i*=1e-7 // t_i converted from 100ns to s
		t_i-=9561628800 // t_i converted from Windows Filetime format (01/01/1601) to Mac Epoch format (01/01/1970)
		
		//Images have a LEEM data block or a LEEMdata array that contains LEEM setup parameters...
		If (LEEMDataBlockSize!=0)
			Make/B/U/O/N=(LEEMDataBlockSize) LEEMDataBlockW
			If(FrameNr==0)
				Fsetpos RefRead,FileHeaderSize+ImageHeaderSize-LEEMDataBlockSize
			//Only applies to 2nd, 3rd,... images of .dav files as FrameNr=0 for .dat files
			Else
				fsetpos RefRead,FileHeaderSize+(FrameNr)*(1024*1024*2)+V_sum+ImageHeaderSizeW[FrameNr]	-LEEMDataBlockSize
			EndIf		
			FBinRead/F=1/U RefRead, LEEMDataBlockW
			imax=LEEMDataBlockSize
		Else
			Make/B/U/O/N=240 LEEMDataBlockW
			LEEMDataBlockW=MyImageHeader.LEEMdata[p]
			imax=240
		Endif 
		Make/B/U/O/N=4 TempW
		//Print LEEMDataBlockW
		
		For(i=0;i<imax;i+=1)
			//108 is the ascii number of the module 'Main'
			If((LEEMDataBlockW[i+1]==77)&&(LEEMDataBlockW[i+2]==97)&&(!PressureLoaded))
				//Sometimes the unit 'Torr' (ascii code 'T' is 84) is inserted between the two 'NULL' characters
				//Sometimes there's a space character (ascii code is 32)!!
				If(LEEMDataBlockW[i+6]==32)
					i+=8
				EndIf
				if(LEEMDataBlockW[i+6]==84)
					i+=11
				Endif
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				Pressure=MyDummySTruct.Float				
//				fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
//				FBinRead/f=4 RefRead, Pressure
				If(Pressure==0 || Pressure>1)	
					Pressure=nan 
				Endif				
				i+=3
				PressureLoaded=1
			EndIf	
				
			//110 is the ascii number of the module FOV but it's also the character 'n'
			If((LEEMDataBlockW[i]==110)&&(!FOVLoaded))
				//To make sure we are not in the 'Main' block or in the  'Emission Cur.'
				If(LEEMDataBlockW[i+1]==50 || LEEMDataBlockW[i+1]==53 || LEEMDataBlockW[i+1]==76 || LEEMDataBlockW[i+1]==49)
					//FOV=LEED
					If(LEEMDataBlockW[i+1]==76)
						FOV="LEED"
						// To reach the end-1 of this block
						i+=9
					Else
						FOV=num2char(LEEMDataBlockW[i+1])
						FOV+=num2char(LEEMDataBlockW[i+2])
						FOV+=num2char(LEEMDataBlockW[i+3])
						FOV+=num2char(LEEMDataBlockW[i+4])
											
						//3 digit FOV:
						If(LEEMDataBlockW[i+6]==42)	
							FOV+=num2char(LEEMDataBlockW[i+5])
							i+=1
						EndIf
						i+=10
					EndIf
				FOVLoaded=1	
				EndIf
			EndIf
			
			//38 is the ascii number of the module 'Start Voltage'
			If((LEEMDataBlockW[i]==38)&&(LEEMDataBlockW[i+2]==116)&&(!SVLoaded))
				i+=16 
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				SV=MyDummySTruct.Float
//				SV=LEEMDataBlockW[i]
//				SV+=2^8*LEEMDataBlockW[i+1]
//				SV+=2^16*LEEMDataBlockW[i+2]
//				SV+=2^24*LEEMDataBlockW[i+3]
				//fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
				//FBinRead/f=4 RefRead, SV
				// To reach the end-1 of this block
				i+=3
				SVLoaded=1
			EndIf
			
			//39 is the ascii number of the module 'Sample Temp.'
			If((LEEMDataBlockW[i]==39)&&(LEEMDataBlockW[i+3]==109)&&(!TempLoaded))
				//File header:15 bytes to reach Temp. data
				i+=15
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				Temp=MyDummySTruct.Float
//				fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
//				FBinRead/f=4 RefRead, Temp
				// To reach the end-1 of this block
				i+=3
				TempLoaded=1
			EndIf
			
			//11 is the ascii number of the module Obj
			If((LEEMDataBlockW[i]==11)&&(LEEMDataBlockW[i+1]==79)&&(!ObjLoaded))
				i+=12
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				Obj=MyDummySTruct.Float
//				fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
//				FBinRead/f=4 RefRead, Obj
				i+=3
				ObjLoaded=1
			EndIf
		EndFor
	
	
	
		Str=StrVarOrDefault("root:TopImageWinName","nop!")
		If(stringmatch(str,"nop!"))
			String/G TopImageWinName
		Else
			Svar TopImageWinName
		EndIf
		TopImageWinName=WinName(0,1,1)
		//print "TopImageWinName", TopImageWinName
		//Create a new Graph window if there's no image/sequence already or if a single image is loaded or if 
		//a new sequence is opened
		If((strlen(TopImageWinName)==0)||(DisplayMode==2)||(stringmatch(TopImageWinName,"*WMContrastAdjustGraph*")))
			Display/T/K=1/W=(0, 0, 512, 512 ); AppendImage/T/G=0 $FrameName; ModifyGraph margin=-1
			//NewImage/K=1/S=0/G=0 $FrameName
			AutoPositionWindow/E/M=1/R=LeemPanel2015
		Else
			Str=ImageNameList(TopImageWinName, "," )
			Str=RemoveEnding(Str, "," )
			RemoveImage/Z/W=$TopImageWinName  $Str
			AppendImage/T/W=$TopImageWinName $FrameName
		Endif
		ModifyImage $FrameName ctab= {*,*,Grays},log=1
		//ModifyImage $FrameName ctab= {MyImageHeader.ColorScaleLow,MyImageHeader.ColorScaleHigh,Grays}
		
		//Reverses the left axis
		SetAxis/A/R left;
		
		fstatus RefRead
		// Add overlayer data : sample temperature, FOV, SV, ... or not (last option used in convert procedure)
		If(stringmatch(OverlayerStr,"No"))
			TextBox/C/N=Title/V=0
			TextBox/C/N=Day/V=0
			TextBox/C/N=Ell/V=0
			TextBox/C/N=SV/V=0
			TextBox/C/N=Temp/V=0
			TextBox/C/N=Press/V=0
			TextBox/C/N=FOV/V=0
			TextBox/C/N=Obj/V=0
		Else
			//Overlay data are already displayed, Textboxes only need to be modified not created
			If(StringMatch(annotationlist(""),"*Day*"))
				If(StringMatch(S_fileName,"*.dav*"))
					TextBox/C/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName+" "+FrameName
				Else
					TextBox/C/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName
				EndIf
				TextBox/C/N=Day/F=0/B=1/A=LT/X=2/Y=93"\\Z10\\K(65280,65280,0)"+secs2date(t_i,0)+" "+secs2time(t_i,1)
				TextBox/C/N=Ell/F=0/B=1/A=LT/X=2/Y=96"\\Z10\\K(65280,65280,0)Elapsed time="+num2str(ElapsedTime)+"s"
				TextBox/C/N=Temp/F=0/B=1/A=LT/X=79/Y=5"\\Z10\\K(65280,65280,0) T\Bsample\M\\Z10="+num2str(Temp)+"C"
				TextBox/C/N=Press/F=0/B=1/A=LT/X=79/Y=2"\\Z10\\K(65280,65280,0) P="+num2str(Pressure)+"Torr"
				TextBox/C/N=FOV/F=0/B=1/A=LT/X=79/Y=93"\\Z10\\K(65280,65280,0) FOV="+FOV//+"m"
				TextBox/C/N=SV/F=0/B=1/A=LT/X=79/Y=90 "\\Z10\\K(65280,65280,0) SV="+num2str(SV)+"V"
				TextBox/C/N=Obj/F=0/B=1/A=LT/X=79/Y=96"\\Z10\\K(65280,65280,0) Obj.="+num2str(Obj)+"mA"
			else
				If(StringMatch(S_fileName,"*.dav*"))
					TextBox/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName+" "+FrameName
				Else
					TextBox/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName
				EndIf
				TextBox/N=Day/F=0/B=1/A=LT/X=2/Y=93"\\Z10\\K(65280,65280,0)"+secs2date(t_i,0)+" "+secs2time(t_i,1)
				TextBox/N=Ell/F=0/B=1/A=LT/X=2/Y=96"\\Z10\\K(65280,65280,0)Elapsed time="+num2str(ElapsedTime)+"s"
				TextBox/N=Temp/F=0/B=1/A=LT/X=79/Y=5"\\Z10\\K(65280,65280,0) T\Bsample\M\\Z10="+num2str(Temp)+"C"
				TextBox/N=Press/F=0/B=1/A=LT/X=79/Y=2"\\Z10\\K(65280,65280,0) P="+num2str(Pressure)+"Torr"
				TextBox/N=FOV/F=0/B=1/A=LT/X=79/Y=93"\\Z10\\K(65280,65280,0) FOV="+FOV//+"m"
				TextBox/N=SV/F=0/B=1/A=LT/X=79/Y=90 "\\Z10\\K(65280,65280,0) SV="+num2str(SV)+"V"
				TextBox/N=Obj/F=0/B=1/A=LT/X=79/Y=96"\\Z10\\K(65280,65280,0) Obj.="+num2str(Obj)+"mA"
			EndIf
		EndIf
				
		//Adjust Image Contrast if "WMContrastAdjustGraph" Window is loaded 
		Str=WinList("*",";","WIN:1")
		if(StringMatch(Str,"*WMContrastAdjustGraph*"))
			SVAR ImGrfName=root:Packages:WMImProcess:ImageContrast:ImGrfName
			ImGrfName=WinName(0,1,1)
			WMContrastButtonProc("Apply")
			
			Str=WaveList("*_CLU",";","")
			KillWaves/Z Str
		EndIf
		
	EndIf		
	//close /A
	Killwaves/Z TempW,LEEMDataBlockW
	return w
End

Function DAVLoadImageHeaderSizes()
	Nvar RefRead
	Nvar FrameNumber;FrameNumber=0//; FrameNumber=FrameNr
	Nvar NFrames, FileHeaderSize, ImageHeaderSize,LEEMDataBlockSize
	
	
	STRUCT UKFileHeader MyFileHeader 
	STRUCT UKImageHeader MyImageHeader
	
	FileHeaderSize=0;LEEMDataBlockSize=0;ImageHeaderSize=0
	
	//For .dav files the appropriate file is already opened from SeqOpen or LoadSingleImage
	fsetpos RefRead,0
	FBinRead RefRead, MyFileHeader
	// File header: 104 bytes and potentially 128 bytes if attachedRecipeSize>0
	if(MyFileHeader.attachedRecipeSize>0)		
		FileHeaderSize=104+128
	else
		FileHeaderSize=104
	Endif
	
	Make/O/N=1 ImageHeaderSizeW
	ImageHeaderSizeW=0
	Do 	
		FStatus RefRead
		Wavestats/Q/M=1 ImageHeaderSizeW
		Fsetpos RefRead,FileHeaderSize+(FrameNumber)*(1024*1024*2)+V_sum	
		FStatus RefRead
		FBinRead RefRead, MyImageHeader
		if(MyImageHeader.LEEMDataVersion>2)
			LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		Else
			LEEMDataBlockSize=0
		Endif
		//Elmitec files can have additional markup blocks hence the offsets of 128 bytes
		If(MyImageHeader.attachedMarkupSize==0)
			//no markups
			ImageHeaderSize= 288+LEEMDataBlockSize
		Else		
			//Markup blocks multiple of 128 bytes after image header	
			ImageHeaderSize= 288+128*((trunc(MyImageHeader.attachedMarkupSize/128))+1)+LEEMDataBlockSize
		EndIf
		//Print/D FrameNumber, FileHeaderSize+(FrameNumber)*(1024*1024*2)+V_sum, V_filePos, V_logEOF
		
		ImageHeaderSizeW[FrameNumber]=ImageHeaderSize
		
		FrameNumber+=1
		Redimension/E=1/N=(FrameNumber+1) ImageHeaderSizeW
		FStatus RefRead
	while((FileHeaderSize+(FrameNumber)*(1024*1024*2)+V_sum+1350)<(V_logEOF))
	NFrames=FrameNumber
	Redimension/E=1/N=(FrameNumber) ImageHeaderSizeW
	FrameNumber=0
End


//Loads and save overlays in dedicated waves Pressure, Time, ...
Function DAVOverlays(FrameNr)
	
	Variable FrameNr
	
	Nvar RefRead, t_i, t_0
	Nvar FrameNumber//; FrameNumber=FrameNr
	String/G FrameName//, FileHeader, ImageHeader
	
	Variable Temp=0, SV=0, Pressure=0, ElapsedTime=0, i=0,imax=0, TempVar
	Wave SampleTempW, PressureW, ElapsedTimeW,StartVoltW
	String Str //FOV="", 
		
	STRUCT UKFileHeader MyFileHeader 
	STRUCT UKImageHeader MyImageHeader
	STRUCT DummyStruct MyDummyStruct
	Nvar FileHeaderSize, ImageHeaderSize, LEEMDataBlockSize
	Variable TempLoaded=0, SVLoaded=0, PressureLoaded=0
	
	fsetpos RefRead,0
	FBinRead RefRead, MyFileHeader
	
	// File header: 104 bytes and potentially 128 bytes if attachedRecipeSize>0
	if(MyFileHeader.attachedRecipeSize>0)		
		FileHeaderSize=104+128
	else
		FileHeaderSize=104
	Endif
	
	TempVar=FrameNr
	
	Fstatus RefRead	
	If(StringMatch(S_filename, "*.dat*"))
		// for .dat sequence we have to open the appropriate file
		//If(FrameNr!=0)
			Svar DATSeqFilenameList
			Close RefRead
			Str=StringFromList(FrameNr,DATSeqFilenameList)
			open/R /P=DATSeqPath RefRead as Str
		//EndIf
		FrameNr=0	
	EndIf
		
	//Only applies to the 1st image of.dav files and .dat files
	If(FrameNr==0)
		fsetpos RefRead,FileHeaderSize
		FBinRead RefRead, MyImageHeader
		if(MyImageHeader.LEEMDataVersion>2)
			LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		Else
			LEEMDataBlockSize=0
		Endif
		//Elmitec files can have additional markup blocks hence the offsets of 128 bytes
		If(MyImageHeader.attachedMarkupSize==0)
			//no markups
			ImageHeaderSize= 288+LEEMDataBlockSize
		Else		
			//Markup blocks multiple of 128 bytes after image header	
			ImageHeaderSize= 288+128*((trunc(MyImageHeader.attachedMarkupSize/128))+1)+LEEMDataBlockSize
		EndIf
	//Only applies to .dav files as FrameNr=0 for .dat files
	Else
		wave ImageHeaderSizeW
		Wavestats/Q/Z/M=1/R=[0,FrameNr-1] ImageHeaderSizeW
		fsetpos RefRead,FileHeaderSize+(FrameNr)*(1024*1024*2)+V_sum	
		FBinRead RefRead, MyImageHeader
		if(MyImageHeader.LEEMDataVersion>2)
			LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		Else
			LEEMDataBlockSize=0
		Endif
	EndIf	
						
	fstatus RefRead
	
	t_i=0				
	
	t_i=MyImageHeader.imagetime.LONG[0]
	t_i+=2^32*MyImageHeader.imagetime.LONG[1]
		
	If(TempVar==0)
		t_0=t_i
		ElapsedTimeW[TempVar]=0
	Else
		ElapsedTimeW[TempVar]=(t_i-t_0)*1e-7 //Elapsed  time (from t_0 to t_i) converted from 100ns to s
	EndIf
	
	//Images have a LEEM data block or a LEEMdata array that contains LEEM setup parameters...
	If (LEEMDataBlockSize!=0)
		Make/B/U/O/N=(LEEMDataBlockSize) LEEMDataBlockW
		If(FrameNr==0)
			Fsetpos RefRead,FileHeaderSize+ImageHeaderSize-LEEMDataBlockSize
		//Only applies to 2nd, 3rd,... images of .dav files as FrameNr=0 for .dat files
		Else
			fsetpos RefRead,FileHeaderSize+(FrameNr)*(1024*1024*2)+V_sum+ImageHeaderSizeW[FrameNr]	-LEEMDataBlockSize
		EndIf		
		FBinRead/F=1/U RefRead, LEEMDataBlockW
		imax=LEEMDataBlockSize
	Else
		Make/B/U/O/N=240 LEEMDataBlockW
		LEEMDataBlockW=MyImageHeader.LEEMdata[p]
		imax=240
	Endif 
	Make/B/U/O/N=4 TempW
	//Print LEEMDataBlockW
	
	For(i=0;i<imax;i+=1)
			//108 is the ascii number of the module 'Main'
			If((LEEMDataBlockW[i+1]==77)&&(LEEMDataBlockW[i+2]==97)&&(!PressureLoaded))
				//Sometimes the unit 'Torr' (ascii code 'T' is 84) is inserted between the two 'NULL' characters
				//Sometimes there's a space character (ascii code is 32)!!
				If(LEEMDataBlockW[i+6]==32)
					i+=8
				EndIf
				if(LEEMDataBlockW[i+6]==84)
					i+=11
				Endif
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				Pressure=MyDummySTruct.Float				
//				fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
//				FBinRead/f=4 RefRead, Pressure
				If(Pressure==0 || Pressure>1)	
					Pressure=nan 
				Endif				
				i+=3
				PressureLoaded=1
				PressureW[TempVar]=Pressure
			EndIf
			
			//38 is the ascii number of the module 'Start Voltage'
			If((LEEMDataBlockW[i]==38)&&(LEEMDataBlockW[i+2]==116)&&(!SVLoaded))
				i+=16 
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				SV=MyDummySTruct.Float
//				SV=LEEMDataBlockW[i]
//				SV+=2^8*LEEMDataBlockW[i+1]
//				SV+=2^16*LEEMDataBlockW[i+2]
//				SV+=2^24*LEEMDataBlockW[i+3]
				//fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
				//FBinRead/f=4 RefRead, SV
				// To reach the end-1 of this block
				i+=3
				SVLoaded=1
				StartVoltW[TempVar]=SV
			EndIf	
			
			//39 is the ascii number of the module 'Sample Temp.'
			If((LEEMDataBlockW[i]==39)&&(LEEMDataBlockW[i+3]==109)&&(!TempLoaded))
				//File header:15 bytes to reach Temp. data
				i+=15
				TempW=LEEMDataBlockW[i+p]
				StructGet/b=3 MyDummySTruct,TempW
				Temp=MyDummySTruct.Float
//				fsetpos RefRead,FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2-(MyImageHeader.LEEMDataVersion)+i
//				FBinRead/f=4 RefRead, Temp
				// To reach the end-1 of this block
				i+=3
				TempLoaded=1
				SampleTempW[TempVar]=Temp
			EndIf
	EndFor
	Killwaves/Z TempW, LEEMDataBlockW
End



//----------------------------------------------------------------------------------------------------------------------------------------------------//
//								DAVBuild & DAVExtract & DAVConvert							//
//----------------------------------------------------------------------------------------------------------------------------------------------------//

//Build a .dav file from a sequence of .dat images sharing the same filename root
Function DAVBuild() 
	
	Nvar RefRead,NFrames
	Svar DATSeqRootFilename, DATSeqFilenameList
	
	Variable /g RefWrite
	Variable i, Flag
	String TmpStr,Str
	STRUCT UKFileHeader MyFileHeader 
		
	
	Str=StrVarOrDefault("root:TopImageWinName","nop!")
	If(stringmatch(str,"nop!"))
		String/G TopImageWinName
	Else
		Svar TopImageWinName
	EndIf
	TopImageWinName=WinName(0,1,1)
	If(strlen(TopImageWinName)!=0)
		KillWindow $TopImageWinName
	Endif
		
	TmpStr=IndexedFile(DATSeqPath,-1,".dav")
	If(StringMatch(TmpStr, "*"+DATSeqRootFilename+"_build.dav*"))
		Deletefile/P=DATSeqPath DATSeqRootFilename+"_build.dav"
	EndIf
	open/P=DATSeqPath RefWrite as DATSeqRootFilename+"_build.dav"

	If(V_flag==0)		
		Print " Building ",DATSeqRootFilename+"_build.dav file"
		fsetpos RefRead,0
		FBinRead RefRead, MyFileHeader
		fsetpos RefWrite,0
		FBinWrite RefWrite, MyFileHeader
		//Print MyFileHeader
		//Close RefRead
		for(i=0; i<NFrames; i=i+1)		
			//Wave 'Wave_i'=DAVLoadFrame(i,0)	
			DAVWriteFrame("Frame"+num2str(i),i,"DAVBuild")//, RefRead,RefWrite)
			//Killwaves/Z 'Wave_i'
			//WriteDATFrameToDAV(i)//,"dat")//, RefRead,RefWrite)
			//Print i
		EndFor
		
		Close /A //RefWrite
		open/P=DATSeqPath/R RefRead DATSeqRootFilename+"_build.dav"
	EndIf
End

//Function WriteDATFrameToDAV(FramePosition)//, RefRead,RefWrite)
//	Variable FramePosition
//	String OptionStr
//	
//	Nvar RefRead,RefWrite
//	Nvar FileHeaderSize, ImageHeaderSize
//	Svar DATSeqFilenameList
//
//	STRUCT UKImageHeader MyImageHeader
// 
//	make/D/O/N=(1024,1024) w
//
//	open/R/Z/P=DATSeqPath RefRead StringFromList(FramePosition,DATSeqFilenameList)
//	
//	Fsetpos RefRead,FileHeaderSize
//	FBinRead RefRead, MyImageHeader	
//	fsetpos RefWrite,FileHeaderSize+(FramePosition)*(ImageHeaderSize+1024*1024*2)
//	FBinWrite RefWrite, MyImageHeader
//		
//	fsetpos RefRead, FileHeaderSize+ImageHeaderSize	
//	FBinRead/f=2 RefRead, w
//	fsetpos RefWrite, FileHeaderSize+ImageHeaderSize*(FramePosition+1)+(FramePosition)*1024*1024*2		
//	FBinWrite/f=2 RefWrite, w
//	close RefRead
//End

//Extract specified images from a .dav file into a new movie file
Function DAVExtract(ctrlName) : ButtonControl
	String ctrlName
	
	Nvar RefRead
	
	Nvar FromVar, ToVar, StepVar
	Svar Message
	
	Variable /g RefWrite
	Variable i, NFramesToExtract,Flag,TempVar
	
//	Prompt FromVar, "From frame: "
//	Prompt ToVar,"To:"
//	Prompt StepVar,"Step:"
//	DoPrompt "Extract sequence from a .dav file",FromVar, ToVar, StepVar
	Flag=FromToStepDialog(1)
	
	NFramesToExtract=(ToVar-FromVar)/StepVar+1
	//print NFramesToExtract
	
	If(Flag==0)
		STRUCT UKFileHeader MyFileHeader 
		Fstatus RefRead
		
		open/F="Data Files (*.dav):.dav;All Files:.*;" RefWrite as StringFromList(0, S_fileName,".")+"_extr.dav"
		
		fsetpos RefRead,0
		fsetpos RefWrite,0
		
		FBinRead RefRead, MyFileHeader
		FBinWrite RefWrite, MyFileHeader
		
		for(i=0; i<NFramesToExtract; i+=1)
		
			TempVar=FromVar+i*StepVar	
			Wave 'Wave_i'=DAVLoadFrame(TempVar,0)			
			
			DAVWriteFrame(NameOfWave('Wave_i'),i,"DAVExtract")//, RefRead,RefWrite)
			Killwaves/Z 'Wave_i'
			//Print i
		EndFor
		
		Close RefWrite
		open/R/F="Data Files (*.dav):.dav;All Files:.*;" RefRead as StringFromList(0, S_fileName,".")+"_extr.dav"
	EndIf
End

Function DAVWriteFrame(FrameName,FramePosition,CallRefName)//, RefRead,RefWrite)
	String FrameName,CallRefName
	Variable FramePosition
	
	Nvar RefRead,RefWrite
	Nvar FileHeaderSize, ImageHeaderSize, LEEMDataBlockSize
	
	Variable FrameNr=str2num(replacestring("Frame",FrameName,""))
	//Variable FrameNr=str2num(replacestring("Frame",NameOfWave('Wave_i+1'),""))
	Variable MarkupInfoSize
	
	STRUCT UKFileHeader MyFileHeader 
	STRUCT UKImageHeader MyImageHeader
	make/D/O/N=(1024,1024) w
			
	fstatus RefRead
	Print "Writing :",FrameName, "in position", FramePosition
	
	If(StringMatch(S_filename, "*.dat*"))
		Svar DATSeqFilenameList
		Close RefRead
		open/R/P=DATSeqPath RefRead as StringFromList(FrameNr,DATSeqFilenameList)
		FrameNr=0
		FrameName="Frame0"
	EndIf
			
// In case DAVWriteFrame is called from DAVBackground MyImageHeader.ColorScaleHigh and Low have to be modified
//	ImageStats /M=1 $FrameName
//	//Print V_min, V_max
//	MyImageHeader.ColorScaleHigh=V_max
//	MyImageHeader.ColorScaleLow=V_min
	//Print MyImageHeader.ColorScaleLow, MyImageHeader.ColorScaleHigh 
	
	if(FrameNr==0)
		fsetpos RefRead,0
		FBinRead RefRead, MyFileHeader
		// File header: 104 bytes and potentially 128 bytes if attachedRecipeSize>0
		if(MyFileHeader.attachedRecipeSize>0)		
			FileHeaderSize=104+128
		else
			FileHeaderSize=104
		Endif
		fsetpos RefRead,FileHeaderSize//+(FrameNr)*(ImageHeaderSize+1024*1024*2)
		FBinRead RefRead, MyImageHeader
		if(MyImageHeader.LEEMDataVersion>2)
			LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		Else
			LEEMDataBlockSize=0
		Endif
		
		fstatus RefWrite
		fsetpos RefWrite,V_logEOF
		//fsetpos RefWrite,FileHeaderSize+(FramePosition)*(ImageHeaderSize+1024*1024*2)
		//Print FileHeaderSize,ImageHeaderSize//, FileHeaderSize+(FramePosition)*(ImageHeaderSize+1024*1024*2)
		FBinWrite RefWrite, MyImageHeader
	
		//Elmitec files can have additional markup blocks hence the offsets of 128 bytes
		If(MyImageHeader.attachedMarkupSize==0)
			//no markups
			ImageHeaderSize= 288+LEEMDataBlockSize
		Else		
			//Markup blocks multiple of 128 bytes after image header	
			MarkupInfoSize=128*((trunc(MyImageHeader.attachedMarkupSize/128))+1)
			ImageHeaderSize= 288+MarkupInfoSize+LEEMDataBlockSize
		EndIf
		Make/B/U/O/N=(MarkupInfoSize) MarkupInfoW
		fsetpos RefRead, FileHeaderSize+ImageHeaderSize-MarkupInfoSize-LEEMDataBlockSize
		FBinRead/F=1/U RefRead, MarkupInfoW
		fstatus RefWrite
		fsetpos RefWrite,V_logEOF
		FBinWrite/F=1/U RefWrite, MarkupInfoW
		if(LEEMDataBlockSize!=0)
			fsetpos RefRead, FileHeaderSize+ImageHeaderSize-LEEMDataBlockSize//*(FrameNr+1)+(FrameNr)*1024*1024*2	
		
			Make/B/U/O/N=(LEEMDataBlockSize) LEEMDataBlockW
			FBinRead/F=1/U RefRead, LEEMDataBlockW
		
			fstatus RefWrite
			fsetpos RefWrite,V_logEOF
			FBinWrite/F=1/U RefWrite,LEEMDataBlockW
		Endif		
		fsetpos RefRead, FileHeaderSize+ImageHeaderSize
	Else
		wave ImageHeaderSizeW
		Wavestats/Q/Z/M=1/R=[0,FrameNr-1] ImageHeaderSizeW
		fsetpos RefRead,FileHeaderSize+(FrameNr)*(1024*1024*2)+V_sum	
		FBinRead RefRead, MyImageHeader
		LEEMDataBlockSize=MyImageHeader.LEEMDataVersion
		fstatus RefWrite
		fsetpos RefWrite,V_logEOF
		FBinWrite RefWrite, MyImageHeader
		
		MarkupInfoSize = MyImageHeader.attachedMarkupSize==0 ? 0 : 128*((trunc(MyImageHeader.attachedMarkupSize/128))+1)
		
		Make/B/U/O/N=(MarkupInfoSize) MarkupInfoW
		fsetpos RefRead, FileHeaderSize+(FrameNr)*1024*1024*2+V_sum+ImageHeaderSizeW[FrameNr]-MarkupInfoSize-LEEMDataBlockSize
		
		FBinRead/F=1/U RefRead, MarkupInfoW
		fstatus RefWrite
		fsetpos RefWrite,V_logEOF
		FBinWrite/F=1/U RefWrite, MarkupInfoW
		
		if(LEEMDataBlockSize!=0)				
			fsetpos RefRead, FileHeaderSize+(FrameNr)*1024*1024*2+V_sum+ImageHeaderSizeW[FrameNr]-LEEMDataBlockSize
		
			Make/B/U/O/N=(LEEMDataBlockSize) LEEMDataBlockW
			FBinRead/F=1/U RefRead, LEEMDataBlockW
		
			fstatus RefWrite
			fsetpos RefWrite,V_logEOF
			FBinWrite/F=1/U RefWrite,LEEMDataBlockW
		Endif
		
		fsetpos RefRead, FileHeaderSize+(FrameNr)*1024*1024*2+V_sum+ImageHeaderSizeW[FrameNr]
	Endif
	
	If(stringmatch(CallRefName,"*PosStartButton*"))	
		FBinWrite/f=2 RefWrite, $FrameName
	Else
		FBinRead/f=2 RefRead, w
		//fstatus RefWrite
		//fsetpos RefWrite,V_logEOF
		//fsetpos RefWrite, FileHeaderSize+ImageHeaderSize*(FramePosition+1)+(FramePosition)*1024*1024*2			
		FBinWrite/f=2 RefWrite, w
	Endif

	Killwaves w, LEEMDataBlockW
End

Function DAVConvert(ctrlName) : ButtonControl
	String ctrlName
	
	Nvar RefRead
	Nvar FromVar, ToVar, StepVar, DurationVar
	Svar FormatStr,OverlayerStr
	Variable i, NFramesToConvert, Flag, FrameRate
	
	Flag=FromToStepDialog(2)
	
	If(Flag==1)
		abort "Convert procedure cancelled"
	Else
	
		NFramesToConvert=trunc((ToVar-FromVar)/StepVar)+1
		FrameRate=NFramesToConvert / DurationVar
		
		If(FrameRate<1)
			 FrameRate=1 
		 Endif
		If(FrameRate>60)
			 FrameRate=60 
		 Endif
		
		If(Stringmatch(FormatStr,".avi"))
			newmovie/A/O/f=(FrameRate)/I
			Print "Converting",NFramesToConvert, "frames to a .avi file at a video rate of", FrameRate,"fr/s"
		Else
			newmovie/L/O/f=(FrameRate)/I
			Print "Converting",NFramesToConvert,"frames to a .mov file at a video rate of",FrameRate,"fr/s"
		EndIf
	
		for(i=0; i<NFramesToConvert; i+=1)
			DAVAddFrame(FromVar+i*StepVar,"Yes")
			DoUpdate
			AddMovieFrame
		EndFor
		closemovie
		OverlayerStr="Yes"	
	EndIf
	
End

//----------------------------------------------------------------------------------------------------------------------------------------------------//
//					DAVSlider DAVSliderButton Proc and DAVAdd								//
//----------------------------------------------------------------------------------------------------------------------------------------------------//


Function DAVsliderProc(name, value, event) : SliderControl
	String name	// name of this slider control
	Variable value	// value of slider
	Variable event	// bit field: bit 0: value set; 1: mouse down, 
				//   2: mouse up, 3: mouse moved		

	Nvar DoesCursorAExist
	String Str
	
	If(strlen(ImageNameList("", ";"))>0)
		//print "cursor=",CsrInfo(A)
		Str=CsrInfo(A)
		if(strlen(Str)>0)
			DoesCursorAExist=1
			Svar XcursorAStr,YcursorAStr
			XcursorAStr=StringByKey("POINT", Str, ":", ";")
			YcursorAStr=StringByKey("YPOINT", Str, ":", ";")
		Else
			DoesCursorAExist=0
		Endif
	EndIf
	
	//Once the slider pointer is moved and released
	If(event==9)
		//No .dav or .dat frame displayed 
		if(!StringMatch(ImageNameList("", ";"),"*Frame*"))
			DAVLoad("DAVLoadButton")
			
		//A .dav or .dat frame is already displayed 
		Else
			DAVAddFrame(value,"Yes")	
		EndIf		
		
		If(Exists("M_ROIMask"))
			If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
				Str=StrVarOrDefault("root:TopImageWinName","nop!")
				If(stringmatch(str,"nop!"))
					String/G TopImageWinName
				Else
					Svar TopImageWinName
				EndIf
				TopImageWinName=WinName(0,1,1)
				//RemoveImage M_ROIMask_boundary
				AppendROIBoundary()
				
			Else
				CalculateAndAppendROIBoundary()	
			EndIf
			
			svar TopImageWaveName
			TopImageWaveName=StringFromList(0, ImageNameList("",";"))
			//print TopImageWaveName
			// Imagestats /R=M_ROIMask $TopImageWaveName
			 //print V_npnts*V_avg
		EndIf		
		
	//		If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
	//	If(event==9)
	//		AppendROIBoundary()
	//	Else
	//		CalculateAndAppendROIBoundary()	
	//	EndIf
	//	print event
	EndIf			
	return 0	// other return values reserved
End

Function DAVSliderButton(ctrlName) : ButtonControl
	String ctrlName
	Nvar NFrames, FrameNumber
	String Str
	
	ControlInfo /W=LeemPanel2015 DAVslider
	If(Stringmatch(ctrlName,"*Right*"))
		If(V_Value<NFrames-1)
			V_Value+=1
			DAVAddFrame(V_Value, "Yes")
			Slider DAVslider value=V_Value, win=LeemPanel2015
			FrameNumber=V_Value
		EndIf
	Else
		If(V_Value>0)
			V_Value-=1	
			DAVAddFrame(V_Value, "Yes")
			Slider DAVslider value=V_Value, win=LeemPanel2015
			FrameNumber=V_Value
		EndIf
	Endif
	
	If(Exists("M_ROIMask"))
		If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
			Str=StrVarOrDefault("root:TopImageWinName","nop!")
			If(stringmatch(str,"nop!"))
				String/G TopImageWinName
			Else
				Svar TopImageWinName
			EndIf
			TopImageWinName=WinName(0,1,1)
			//RemoveImage M_ROIMask_boundary
			AppendROIBoundary()
			
		Else
			CalculateAndAppendROIBoundary()	
		EndIf
		
		svar TopImageWaveName
		TopImageWaveName=StringFromList(0, ImageNameList("",";"))
		//print TopImageWaveName
		// Imagestats /R=M_ROIMask $TopImageWaveName
		 //print V_npnts*V_avg
	EndIf

End

//Adds a Frame to the top most graph
Function/WAVE DAVAddFrame(FrameNr, Rmve)
	Variable FrameNr
	String Rmve
	
	Svar OverlayerStr
	String/G TopImageWaveName=""
	String Str
		
	Str=StrVarOrDefault("root:TopImageWinName","nop!")
	If(stringmatch(str,"nop!"))
		String/G TopImageWinName
	Else
		Svar TopImageWinName
	EndIf
	TopImageWinName=WinName(0,1,1)

	If(stringmatch(Rmve,"Yes"))
		TopImageWaveName=StringFromList(0, ImageNameList("",";"))
		RemoveImage $TopImageWaveName
		KillWaves/Z $TopImageWaveName
		If(exists(TopImageWaveName+"_CLU"))
			KillWaves/Z $TopImageWaveName+"_CLU"
		EndIf
		If(StringMatch(ImageNameList("", ";"),"*M_ROIMask_boundary*"))
			RemoveImage M_ROIMask_boundary
		EndIf
	EndIf
	
	Wave w=DAVLoadFrame(FrameNr,1)	
	//Print "Adding Frame", FrameNr
	
	
//	Nvar FrameNumber; FrameNumber=FrameNr
//	Nvar FileHeaderSize, ImageHeaderSize
//	Nvar RefRead, t_i, t_0
//	t_i=0
//	Svar FrameName//, FileHeader, ImageHeader
//	
//	Variable Temp=0, SV=0, Pressure=0, ElapsedTime=0,i,Obj=0
//	String FOV=""
//		
//	STRUCT UKFileHeader MyFileHeader 
//	STRUCT UKImageHeader MyImageHeader
//
//			
//	fsetpos RefRead,0
//	FBinRead RefRead, MyFileHeader
//	// File header: 104 bytes and potentially 128 bytes if attachedRecipeSize>0
//	if(MyFileHeader.attachedRecipeSize>0)		
//		FileHeaderSize=104+128
//	else
//		FileHeaderSize=104
//	Endif
//	Fstatus RefRead
//	
//	If(StringMatch(S_filename, "*.dat*"))
//		// For .dat image sequence we have to open the appropriate file
//		If(FrameNr!=0)
//			Svar DATSeqFilenameList
//			Close/a
//			Str=StringFromList(FrameNr,DATSeqFilenameList)
//			open/R /P=DATSeqPath RefRead as Str
//		EndIf
//		FrameNr=0
//	Endif
//		
//	fsetpos RefRead,FileHeaderSize
//	FBinRead RefRead, MyImageHeader
//	
//	//Elmitec files can have additional markup blocks hence the offsets of 128 bytes
//	If(MyImageHeader.attachedMarkupSize==0)
//		//no markups
//		ImageHeaderSize= 288+MyImageHeader.LEEMDataVersion
//	Else		
//		//Markup blocks multiple of 128 bytes after image header	
//		ImageHeaderSize= 288+128*((trunc(MyImageHeader.attachedMarkupSize/128))+1)+MyImageHeader.LEEMDataVersion
//	EndIf
//	
//	fsetpos RefRead, FileHeaderSize+ImageHeaderSize*(FrameNr+1)+(FrameNr)*1024*1024*2	
//	FrameName="Frame"+num2str(FrameNr)
//	Print FrameName
//	make/D/O/N=(1024,1024) $FrameName /WAVE=w	
//	
//	FBinRead/f=2 RefRead, w
//	
//	
//			
//	
//	t_i=MyImageHeader.imagetime.LONG[0]
//	t_i+=2^32*MyImageHeader.imagetime.LONG[1]
//		
//	If(FrameNr==0)
//		t_0=t_i
//	Else
//		If(t_0==0)			
//			ElapsedTime=0
//		Else
//			ElapsedTime=(t_i-t_0)*1e-7 //Elapsed  time (from t_0 to t_i) converted from 100ns to s
//		EndIf
//	EndIf
//	
//	t_i*=1e-7 // t_i converted from 100ns to s
//	t_i-=9561628800 // t_i converted from Windows Filetime format (01/01/1601) to Mac Epoch format (01/01/1970)
//	
//	For(i=0;i<240;i+=1)
//		If(MyImageHeader.LEEMdata[i]==39)
//			fsetpos RefRead,104+28+i+15+(FrameNr)*(288+1024*1024*2)
//			FBinRead/f=4 RefRead, Temp
//			//Print "Temp i=", i
//			i+=18
//		EndIf
//		If(MyImageHeader.LEEMdata[i]==38)
//			fsetpos RefRead,104+28+i+16+(FrameNr)*(288+1024*1024*2)
//			FBinRead/f=4 RefRead, SV
//			//Print "SV i=", i
//			i+=19
//		EndIf
//		//110 is the ascii number of the module FOV but it's also the character 'n'
//		If(MyImageHeader.LEEMdata[i]==110)
//			//To make sure we are not in the 'Emission Cur.' block or in the 'Main'
//			If(MyImageHeader.LEEMdata[i+1]!=32 && MyImageHeader.LEEMdata[i+1]!=0)
//				//FOV=LEED
//				If(MyImageHeader.LEEMdata[i+1]==76)
//					FOV="LEED"
//					// To reach the end-1 of this block
//					i+=9
//				Else
//					FOV=num2char(MyImageHeader.LEEMdata[i+1])
//					FOV+=num2char(MyImageHeader.LEEMdata[i+2])
//					FOV+=num2char(MyImageHeader.LEEMdata[i+3])
//					FOV+=num2char(MyImageHeader.LEEMdata[i+4])
//					//Print "FOV i=",i
//					//print "LEEM", MyImageHeader.LEEMdata[i+1]
//											
//					//3 digit FOV:
//					If(MyImageHeader.LEEMdata[i+6]==42)	
//						FOV+=num2char(MyImageHeader.LEEMdata[i+5])
//						//Print "FOV i=",i
//						//print "LEEM", MyImageHeader.LEEMdata[i+1]
//						i+=1
//					EndIf
//					i+=10
//				EndIf	
//			EndIf
//		EndIf
//		//106 is the ascii number of the module 'Main'
//		If(MyImageHeader.LEEMdata[i]==106)
//			//Sometimes the unit 'Torr' (ascii code 'T' is 84) is inserted between the two 'NULL' characters
//			//Sometimes there's a space character (ascii code is 32)!!
//			If(MyImageHeader.LEEMdata[i+6]==32)
//				fsetpos RefRead,104+28+i+8+(FrameNr)*(288+1024*1024*2)
//				FBinRead/f=4 RefRead, Pressure
//				i+=11
//			elseif(MyImageHeader.LEEMdata[i+6]==84)
//				fsetpos RefRead,104+28+i+11+(FrameNr)*(288+1024*1024*2)
//				FBinRead/f=4 RefRead, Pressure
//				i+=14
//			Endif
//		EndIf	
//			
//		//11 is the ascii number of the module Obj
//		If(MyImageHeader.LEEMdata[i]==11)
//			fsetpos RefRead,104+28+i+12+(FrameNr)*(288+1024*1024*2)
//			FBinRead/f=4 RefRead, Obj
//			i+=5
//		EndIf
//	EndFor
//
//
//	fstatus RefRead
//	
//	AppendImage/T/W=$TopImageWinName $FrameName
//	ModifyImage $FrameName ctab= {*,*,Grays},log=1
//	//ModifyImage $FrameName ctab= {MyImageHeader.ColorScaleLow,MyImageHeader.ColorScaleHigh,Grays}
//	//Print MyImageHeader.ColorScaleLow, MyImageHeader.ColorScaleHigh
//	
//	//AppendImage $FrameName
//	SetAxis/A/R left;
//	
//	// Add overlayer data : sample temperature, FOV, SV, ...
//	If(stringmatch(OverlayerStr,"No"))
//		TextBox/C/N=Title/V=0
//		TextBox/C/N=Day/V=0
//		TextBox/C/N=Ell/V=0
//		TextBox/C/N=SV/V=0
//		TextBox/C/N=Temp/V=0
//		TextBox/C/N=Press/V=0
//		TextBox/C/N=FOV/V=0
//		TextBox/C/N=Obj/V=0
//	Else
//		If(StringMatch(S_fileName,"*.dav*"))
//			TextBox/C/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName+" "+FrameName
//		Else
//			TextBox/C/V=1/N=Title/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)"+S_fileName
//		EndIf
//		TextBox/N=Day/V=1/C"\\Z10\\K(65280,65280,0)"+secs2date(t_i,0)+" "+secs2time(t_i,1)
//		TextBox/N=Ell/V=1/C"\\Z10\\K(65280,65280,0)Elapsed time="+num2str(ElapsedTime)+"s"
//		TextBox/N=Temp/V=1/C"\\Z10\\K(65280,65280,0) T\Bsample\M\\Z10="+num2str(Temp)+"C"
//		TextBox/N=Press/V=1/C"\\Z10\\K(65280,65280,0) P="+num2str(Pressure)+"Torr"
//		TextBox/N=FOV/F=0/B=1/A=LT/X=79/Y=93/C"\\Z10\\K(65280,65280,0) FOV="+FOV//+"m"
//		TextBox/N=SV/F=0/B=1/A=LT/X=79/Y=90/C"\\Z10\\K(65280,65280,0) SV="+num2str(SV)+"V"
//		TextBox/N=Obj/F=0/B=1/A=LT/X=79/Y=96/C"\\Z10\\K(65280,65280,0) Obj.="+num2str(Obj)+"mA"
//	EndIf
//	
//	//Adjust Image Contrast if "WMContrastAdjustGraph" Window is loaded 
//	Str=WinList("*",";","WIN:1")
//	if(StringMatch(Str,"*WMContrastAdjustGraph*"))
//		SVAR ImGrfName=root:Packages:WMImProcess:ImageContrast:ImGrfName
//		ImGrfName=WinName(0,1,1)
//		WMContrastButtonProc("Apply")	
//				 
//		//Kill *_CLU waves
//		variable b=0
//		do
//			Str= StringFromList(b,WaveList("*_CLU",",",""),",")
//			if( strlen(Str) == 0 )
//				break
//			endif
//			Wave w=$Str
//			//print nameofwave(w)
//			KillWaves/Z w
//			b += 1
//		while (1)	// exit is via break statement		
//	EndIf
	
	//Add cursor A if it was displayed before calling DAVAddFrame function
	Nvar DoesCursorAExist
	If(DoesCursorAExist==1)
		Svar XcursorAStr,YcursorAStr,TopImageWinName,TopImageWaveName
		TopImageWaveName=StringFromList(0, ImageNameList("",";"))
		Wave XPos,YPos, SampleWave
		WaveStats/Q/Z/M=1 SampleWave
		//Print TopImageWinName,TopImageWaveName,V_npnts-1,XPos[V_npnts-1],YPos[V_npnts-1]
		//cursor/C=(65535,0,0)/I/H=1/W=$TopImageWinName A, $TopImageWaveName,XPos[V_npnts-1],YPos[V_npnts-1]
		cursor/C=(65535,0,0)/I/H=1/W=$TopImageWinName A, $TopImageWaveName,str2num(XcursorAStr),str2num(YcursorAStr)
	EndIf
	
	return w
End
	
//----------------------------------------------------------------------------------------------------------------------------------------------------------//
//										Loads .mov Movies										//
//----------------------------------------------------------------------------------------------------------------------------------------------------------//


//Function LoadMovie(B_Struct) : ButtonControl
//	STRUCT WMButtonAction &B_Struct
	//print B_Struct.eventCode
	//print B_Struct.ctrlName
Function LoadMOV(ctrlName) : ButtonControl
	String ctrlName
	
	Variable/G FrameIndex=0
	//Variable V_flag
	close/a
	Setdatafolder root:
	PlayMovie/W=(1,1,201,201)/Z
	//print V_flag
	If(!V_flag)
		Nvar MovFileLoaded
		MovFileLoaded=1
		PlayMovieAction gotoBeginning, extract
		
		If(WaveExists(M_MovieFrame))
			ImageTransform rgb2gray M_MovieFrame
			duplicate/O M_RGB2Gray, Frame0
			KillWaves M_RGB2Gray
		Endif
		
		DoWindow/K Frame
		//AppendImage/W=Frame Frame0
		NewImage  /N=Frame /G=1 /K=1 /S=0 Frame0
		showinfo
		//ModifyGraph width=200*dimsize(Frame0,0)/dimsize(Frame0,1),height=200//width=719,height=683
		AutoPositionWindow/E/M=0
		
		TextBox/N=TextBoxFrameName/A=LT/X=2/Y=2/S=3/G=(0,0,65280) "\\Z11\K(0,0,65280)Frame0"
	EndIf
End

//----------------------------------------------------------------------------------------------------------------------------------------------------//
//								 Dialog function												//
//----------------------------------------------------------------------------------------------------------------------------------------------------/

Function FromToStepDialog(a)
	Variable a
	
	Nvar FromVar, ToVar, StepVar, DurationVar,PolyOrderVar
	Svar YesNoStr, OptionStr, BlackStr,ManualStr, FormatStr, OverlayerStr
	
	Variable FromV=0
	Variable ToV=10
	Variable StepV=1, Thresh, DurationV=10,PolyOrderV=2
	String Mess,YesNoS, OptionS="Nothing", BlackS, ManualS, FormatS,OverlayerS
	
	Prompt FromV, "From frame: "
	Prompt ToV,"To:"
	Prompt StepV,"Step:"
	
	switch(a)												// numeric switch
		case 1:
			Mess="Extract sequence from a .dav file"
			DoPrompt Mess,FromV, ToV, StepV
			FromVar=FromV;ToVar=ToV;StepVar=StepV
			break
			
		case 2:
			Mess="Convert sequence to .mov or .avi format"
			Prompt FormatS,"Movie file Format?", popup ".avi;.mov"
			Prompt DurationV,"Movie Duration (s)?"
			Prompt OverlayerS,"Include overlayer data?", popup "Yes;No"
			DoPrompt/HELP="Frame rate =[# of frames to convert]/[Movie duration]" Mess,FromV, ToV, StepV,FormatS,DurationV,OverlayerS
			FromVar=FromV;ToVar=ToV;StepVar=StepV
			FormatStr=FormatS; DurationVar = DurationV;OverlayerStr=OverlayerS
			break
			
		case 3:
			Mess="Drift Correction"
			Prompt ManualS,"Manual Drift Correction?", popup "Yes;No"
			DoPrompt Mess,FromV, ToV, StepV,ManualS
			If(stringmatch(ManualS,"No"))
				Prompt YesNoS,"Use ROI mask?", popup "Yes;No"
				DoPrompt Mess,YesNoS
				YesNoStr=YesNoS
			EndIf
			FromVar=FromV;ToVar=ToV;StepVar=StepV;ManualStr=ManualS
			break
			
		case 4:
			Mess="Particle Analysis"
			Prompt YesNoS,"Use ROI mask?", popup "Yes;No"
			Prompt OptionS,"Thresholding Method?", popup "Manual;Iterative"
			Prompt BlackS,"Black Particles ?", popup "Yes;No"
			DoPrompt Mess,FromV, ToV, StepV,YesNoS,OptionS, BlackS
			FromVar=FromV;ToVar=ToV;StepVar=StepV;YesNoStr=YesNoS
			OptionStr=OptionS; BlackStr=BlackS
			break
			
		case 5:
			Mess="ROI averaged Intensity"
			DoPrompt/HELP="Evolution of the average intensity of a region of interest (ROI)" Mess,FromV, ToV, StepV
			FromVar=FromV;ToVar=ToV;StepVar=StepV
			break
			
		case 6:
			Mess="Remove Background"
			Prompt OptionS,"Reference Image (M_RemovedBackground)?", popup "No;Yes"
			Prompt PolyOrderV, "Background polynomial Order"
			DoPrompt/HELP="Removes a Polynomial background determine from a region of interest (ROI) or a Reference image to be named M_RemovedBackground" Mess,FromV, ToV, StepV,OptionS,PolyOrderV
			OptionStr=OptionS,FromVar=FromV;ToVar=ToV;StepVar=StepV;PolyOrderVar=PolyOrderV
			break
			
		case 7:
			Mess="ROI Workfunction Measurements"
			DoPrompt/HELP="Workfunction 2D Map of a region of interest (ROI)" Mess,FromV, ToV, StepV
			FromVar=FromV;ToVar=ToV;StepVar=StepV
			break						
	endswitch
	
	If(stringmatch(OptionS,"Manual"))
		Variable/G Threshold
		
		Prompt Thresh,"Threshold value?"
		DoPrompt "Particle Analysis", Thresh
		Threshold=Thresh
	EndIf
	
	Return V_flag
End

//----------------------------------------------------------------------------------------------------------------------------------------------------------//
//										Loads Single Pictures										//
//----------------------------------------------------------------------------------------------------------------------------------------------------------//

Function LoadSingleImage(ctrlName) : ButtonControl
	String ctrlName
	
	String/G FileName
	String/G TopImageWinName
	Svar TopImageWaveName
	String Str
	
	//newpath/Q/O MyPath, "C:Users:Fabien:Documents:Recherche:Dmouillage:LEEM"
	close/a
	Setdatafolder root:
	ControlInfo ImageLoadPopup
	
	// Loading Elmitec .dat files	
	if(stringmatch(S_value,"*dat*"))
		Variable/G RefRead=0,NFrames=0
		open/R /F="Data Files (*.dat):.dat;All Files:.*;" RefRead
		fstatus RefRead
		If(V_flag==0)
			Abort "Open procedure failed"
		else	
			Print " Loading Image "+S_path+S_fileName
			
			Str=StrVarOrDefault("root:TopImageWinName","nop!")
			If(stringmatch(str,"nop!"))
				String/G TopImageWinName
			Else
				Svar TopImageWinName
			EndIf
			TopImageWinName=WinName(0,1,1)
//			Print TopImageWinName
//			If((strlen(TopImageWinName)!=0)&&(!stringmatch(TopImageWinName,"*WMContrastAdjustGraph*")))
//				KillWindow $TopImageWinName
//			Endif
			
			Str=StrVarOrDefault("root:DATSeqFilenameList","nop!")
			If(stringmatch(str,"nop!"))
				String/G DATSeqFilenameList
			Else
				Svar DATSeqFilenameList
			EndIf
			DATSeqFilenameList=""
			
			Slider DAVslider, disable=2, win=LeemPanel2015
			Button DAVSliderButtonRight, disable=2, win=LeemPanel2015
			Button DAVSliderButtonLeft, disable=2, win=LeemPanel2015
			
			DAVLoadFrame(0,2)
		EndIf
	//Loading other image formats			
	Else
		ImageLoad/O /T=$S_value//P=PathToEchantillons
		If(V_flag==1)
			Print " Loading Image "+S_path+S_fileName
			FileName=S_fileName
			TopImageWaveName=StringFromList(0,S_WaveNames) // i.e. Wave0
			NewImage  /G=1 /K=1 /S=0 $TopImageWaveName
			TopImageWinName=WinName(0,1,1) // i.e. Graph0
			showinfo
			//ModifyGraph width=500*dimsize($TopImageWaveName,0)/dimsize($TopImageWaveName,1),height=500//width=719,height=683
			TextBox/C/N=TextBoxFileName/A=MC "\\Z11\K(0,0,65280)"+FileName
			TextBox/C/N=TextBoxFileName/X=-32.99/Y=45.60
		Else
			Abort "Open procedure failed"
		EndIf
	EndIf
End